@@ -1,7 +1,7 @@
 /* ay.h: AY-8-3912 routines
-   Copyright (c) 1999-2001 Philip Kendall
+   Copyright (c) 1999-2003 Philip Kendall
 
-   $Id: ay.h,v 1.10 2001/12/08 00:36:47 pak21 Exp $
+   $Id: ay.h,v 1.11 2003/07/11 11:03:25 pak21 Exp $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -27,19 +27,17 @@
 #ifndef FUSE_AY_H
 #define FUSE_AY_H
 
-#ifndef FUSE_TYPES_H
-#include "types.h"
-#endif			/* #ifndef FUSE_TYPES_H */
+#include <libspectrum.h>
 
 typedef struct ayinfo {
   int present;
   int current_register;
-  BYTE registers[16];
+  libspectrum_byte registers[16];
 } ayinfo;
 
-BYTE ay_registerport_read(WORD port);
-void ay_registerport_write(WORD port, BYTE b);
+libspectrum_byte ay_registerport_read( libspectrum_word port );
+void ay_registerport_write( libspectrum_word port, libspectrum_byte b );
 
-void ay_dataport_write(WORD port, BYTE b);
+void ay_dataport_write( libspectrum_word port, libspectrum_byte b );
 
 #endif			/* #ifndef FUSE_AY_H */
