@@ -1,7 +1,8 @@
 /* joystick.h: Joystick emulation support
    Copyright (c) 2001-2003 Russell Marks, Philip Kendall
+   Copyright (c) 2003 Darren Salt
 
-   $Id: joystick.h,v 1.3 2003/07/11 11:03:25 pak21 Exp $
+   $Id: joystick.h,v 1.4 2003/10/07 13:49:00 pak21 Exp $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -29,8 +30,20 @@
 
 #include <libspectrum.h>
 
-libspectrum_byte joystick_kempston_read( libspectrum_word port );
-void joystick_kempston_write( libspectrum_word port, libspectrum_byte b );
-libspectrum_byte joystick_timex_read( libspectrum_word port, int which );
+/* Number of joysticks known about & initialised */
+extern int joysticks_supported;
+
+/* Init/shutdown functions. Errors aren't important here */
+void fuse_joystick_init( void );
+void fuse_joystick_end( void );
+
+/* Default read function (returns data in Kempston format) */
+libspectrum_byte joystick_default_read( libspectrum_word port,
+					libspectrum_byte which );
+
+/* Interface-specific read functions */
+libspectrum_byte joystick_kempston_read ( libspectrum_word port );
+libspectrum_byte joystick_timex_read ( libspectrum_word port,
+				       libspectrum_byte which );
 
 #endif			/* #ifndef FUSE_JOYSTICK_H */
