@@ -1,7 +1,7 @@
 /* keyboard.c: Routines for dealing with the Spectrum's keyboard
    Copyright (c) 1999-2000 Philip Kendall
 
-   $Id: keyboard.c,v 1.9 2002/02/09 00:33:38 pak21 Exp $
+   $Id: keyboard.c,v 1.15 2003/07/11 11:03:25 pak21 Exp $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -26,26 +26,28 @@
 
 #include <config.h>
 
+#include <libspectrum.h>
+
+#include "ui/ui.h"
 #include "keyboard.h"
-#include "rzx.h"
-#include "types.h"
+#include "scld.h"
 
 /* What to return if no keys are pressed; depends on the last byte
    output to the ULA; see CSS FAQ | Technical Information | Port #FE
    for full details */
-BYTE keyboard_default_value;
+libspectrum_byte keyboard_default_value;
 
 /* Bit masks for each of the eight keyboard half-rows; `AND' the selected
    ones of these with `keyboard_default_value' to get the value to return
 */
-BYTE keyboard_return_values[8];
+libspectrum_byte keyboard_return_values[8];
 
 /* When each Spectrum key is pressed, twiddle this {port,bit} pair
    in `keyboard_return_values'. 
 */
 typedef struct keyboard_key_info {
   keyboard_key_name key;
-  int port; BYTE bit;
+  int port; libspectrum_byte bit;
 } keyboard_key_info;
 
 static keyboard_key_info keyboard_data[] = {
@@ -106,26 +108,13 @@ void fuse_keyboard_init(void)
   keyboard_release_all();
 }
 
-BYTE keyboard_read(BYTE porth)
+libspectrum_byte
+keyboard_read( libspectrum_byte porth )
 {
-  BYTE data=keyboard_default_value; int i;
-
-  /* If we're playing back a .rzx file, start with all bits one. If we're
-     not, start with bit 6 possibly unset */
-  data = rzx_playback ? 0xff : keyboard_default_value;
+  libspectrum_byte data = keyboard_default_value; int i;
 
   for( i=0; i<8; i++,porth>>=1 ) {
-    if(! (porth&0x01) ) {
-
-      /* If we're playing back a .rzx file, use the data from that;
-	 otherwise return what keys are actually pressed */
-      if( rzx_playback ) {
-	data &= rzx.frames[ rzx_current_frame ].keyboard[i];
-      } else {
-	data &= keyboard_return_values[i];
-      }
-
-    }
+    if(! (porth&0x01) ) data &= keyboard_return_values[i];
   }
 
   return data;
@@ -170,3 +159,17 @@ int keyboard_release_all( void )
 
   return 0;
 }
+
+const keysyms_key_info*
+keysyms_get_data( unsigned keysym )
+{
+  const keysyms_key_info *ptr;
+
+  for( ptr=keysyms_data; ptr->keysym; ptr++ ) {
+    if( keysym == ptr->keysym ) {
+      return ptr;
+    }
+  }
+
+  return 0;
+}
