@@ -1,7 +1,7 @@
 /* printer.c: Printer support
    Copyright (c) 2001-2003 Ian Collier, Russell Marks, Philip Kendall
 
-   $Id: printer.c,v 1.14 2003/07/11 11:03:25 pak21 Exp $
+   $Id: printer.c,v 1.17 2003/11/14 11:46:46 pak21 Exp $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -37,9 +37,9 @@
 
 #include "fuse.h"
 #include "machine.h"
+#include "memory.h"
 #include "printer.h"
 #include "settings.h"
-#include "spectrum.h"
 #include "ui/ui.h"
 
 static int printer_graphics_enabled=0;
@@ -285,7 +285,7 @@ chars+=256*readbyte_internal(SYSV_CHARS+1);
 memset(charset,0,sizeof(charset));
 ptr=charset+32*8;
 for(f=32*8;f<128*8;f++)
-  *ptr++=readbyte(chars+f);
+  *ptr++=readbyte_internal(chars+f);
 
 for(x=0;x<32;x++)
   {
@@ -375,6 +375,8 @@ frames++;
  */
 libspectrum_byte printer_zxp_read(libspectrum_word port GCC_UNUSED)
 {
+if(!settings_current.printer)
+  return(0xff);
 if(!printer_graphics_enabled)
   return(0xff);
 
@@ -419,6 +421,8 @@ else
 
 void printer_zxp_write(libspectrum_word port GCC_UNUSED,libspectrum_byte b)
 {
+if(!settings_current.printer)
+  return;
 if(!zxpspeed)
   {
   if(!(b&4))
@@ -611,6 +615,8 @@ old_on=on;
 
 libspectrum_byte printer_parallel_read(libspectrum_word port GCC_UNUSED)
 {
+if(!settings_current.printer)
+  return(0xff);
 /* bit 0 = busy. other bits high? */
 
 return(0xfe);	/* never busy */
@@ -620,6 +626,8 @@ return(0xfe);	/* never busy */
 void printer_parallel_write(libspectrum_word port GCC_UNUSED,
 			    libspectrum_byte b)
 {
+if(!settings_current.printer)
+  return;
 parallel_data=b;
 }
 
