@@ -1,7 +1,7 @@
 /* rzx.c: .rzx files
    Copyright (c) 2002-2003 Philip Kendall
 
-   $Id: rzx.c,v 1.44.4.3 2003/09/23 12:22:15 pak21 Exp $
+   $Id: rzx.c,v 1.56.2.1 2004/02/06 16:55:29 pak21 Exp $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -26,12 +26,13 @@
 
 #include <config.h>
 
-#include <errno.h>
 #include <math.h>
-#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <sys/time.h>
+
+#ifdef WIN32
+#include <windows.h>
+#endif				/* #ifdef WIN32 */
 
 #include "event.h"
 #include "fuse.h"
@@ -90,7 +91,7 @@ libspectrum_rzx_dsa_key rzx_key = {
 };
 
 /* The time we started recording this RZX file */
-static struct timeval start_time;
+static timer_type start_time;
 
 /* How many 1/50ths of a second do we expect to have elapsed since we
    started recording? */
@@ -154,15 +155,23 @@ int rzx_start_recording( const char *filename, int embed_snapshot )
   /* Note that we're recording */
   rzx_recording = 1;
 
-  ui_menu_activate_recording( 1 );
+  ui_menu_activate( UI_MENU_ITEM_RECORDING, 1 );
 
   if( settings_current.competition_mode ) {
 
+    if( !libspectrum_gcrypt_version() )
+      ui_error( UI_ERROR_INFO,
+		"gcrypt not available: recording will NOT be signed" );
+
     expected_time = 0;
     settings_current.emulation_speed = 100;
-    timer_count = 0;
+    timer_count = 0.0;
     rzx_competition_mode = 1;
     
+  } else {
+
+    rzx_competition_mode = 0;
+
   }
     
   return 0;
@@ -178,7 +187,7 @@ int rzx_stop_recording( void )
   /* Stop recording data */
   rzx_recording = 0;
 
-  ui_menu_activate_recording( 0 );
+  ui_menu_activate( UI_MENU_ITEM_RECORDING, 0 );
 
   libspectrum_creator_set_competition_code(
     fuse_creator, settings_current.competition_code
@@ -293,8 +302,8 @@ start_playback( libspectrum_rzx *rzx, libspectrum_snap *snap )
     error = snapshot_copy_from( snap ); if( error ) return error;
   }
   
-  /* Interrupts will now be generated by the RZX code */
-  error = event_remove_type( EVENT_TYPE_INTERRUPT );
+  /* End of frame will now be generated by the RZX code */
+  error = event_remove_type( EVENT_TYPE_FRAME );
   if( error ) return error;
 
   /* We're now playing this RZX file */
@@ -305,7 +314,7 @@ start_playback( libspectrum_rzx *rzx, libspectrum_snap *snap )
   rzx_playback = 1;
   counter_reset();
 
-  ui_menu_activate_recording( 1 );
+  ui_menu_activate( UI_MENU_ITEM_RECORDING, 1 );
 
   return 0;
 }
@@ -316,15 +325,15 @@ int rzx_stop_playback( int add_interrupt )
 
   rzx_playback = 0;
 
-  ui_menu_activate_recording( 0 );
+  ui_menu_activate( UI_MENU_ITEM_RECORDING, 0 );
 
-  /* We've now finished with the RZX file, so add an interrupt event
-     back in if we've been requested to do so; we don't if we just run
-     out of frames, as this occurs just before a normal interrupt will
-     do this for us */
+  /* We've now finished with the RZX file, so add an end of frame
+     event if we've been requested to do so; we don't if we just run
+     out of frames, as this occurs just before a normal end of frame
+     and everything works normally as rzx_playback is now zero again */
   if( add_interrupt ) {
     error = event_add( machine_current->timings.tstates_per_frame,
-		       EVENT_TYPE_INTERRUPT );
+		       EVENT_TYPE_FRAME );
     if( error ) return error;
   }
 
@@ -356,9 +365,6 @@ static int recording_frame( void )
      speed */
   if( rzx_competition_mode ) {
 
-    struct timeval current_time;
-    float elapsed_time;
-
     expected_time++;
 
     /* We wait one second before starting time measurements at all.
@@ -367,50 +373,43 @@ static int recording_frame( void )
        sound, the first few frames run fast as the sound buffers fill
        up) */
     if( expected_time == 50 ) {
-      
-      error = gettimeofday( &start_time, NULL );
+
+      error = timer_get_real_time( &start_time );
       if( error ) {
-	ui_error(
-          UI_ERROR_INFO,
-	  "couldn't get start time: %s: stopping competition mode RZX recording",
-	  strerror( errno )
-	);
+	ui_error( UI_ERROR_ERROR, "stopping competition mode recording" );
 	rzx_stop_recording();
-	return 1;
+	return error;
       }
-
-
+      
     /* Measuring small time intervals will be inaccurate, so don't
        start checking for speed violations until at least one second
        of measured time has elapsed (ie two seconds into recording) */
     } else if( expected_time > 100 ) {
 
-      float seconds;
+      timer_type current_time;
+      float elapsed_seconds, expected_seconds;
 
-      error = gettimeofday( &current_time, NULL );
+      error = timer_get_real_time( &current_time );
       if( error ) {
-	ui_error(
-          UI_ERROR_INFO,
-	  "couldn't get time: %s: stopping competition mode RZX recording",
-	  strerror( errno )
-	);
+	ui_error( UI_ERROR_ERROR, "stopping competition mode recording" );
 	rzx_stop_recording();
-	return 1;
+	return error;
       }
 
       /* -1 to account for the fact we don't time the first second */
-      seconds = ( expected_time * 0.02 ) - 1;
+      expected_seconds = ( expected_time * 0.02 ) - 1;
 
-      elapsed_time =   current_time.tv_sec  - start_time.tv_sec +
-	             ( current_time.tv_usec - start_time.tv_usec ) / 1000000.0;
-      if( fabs( seconds / elapsed_time - 1 ) > SPEED_TOLERANCE ) {
+      elapsed_seconds =
+	timer_get_time_difference( &current_time, &start_time );
+
+      if( fabs( expected_seconds / elapsed_seconds - 1 ) > SPEED_TOLERANCE ) {
 
 	rzx_stop_recording();
 
 	ui_error(
 	  UI_ERROR_INFO,
 	  "emulator speed is %d%%: stopping competition mode RZX recording",
-	  (int)( 100 * ( seconds / elapsed_time ) )
+	  (int)( 100 * ( expected_seconds / elapsed_seconds ) )
 	);
 
       }
