@@ -1,7 +1,7 @@
 /* snapshot.c: snapshot handling routines
    Copyright (c) 1999-2003 Philip Kendall
 
-   $Id: snapshot.c,v 1.74 2003/07/11 11:03:25 pak21 Exp $
+   $Id: snapshot.c,v 1.75 2003/11/07 14:07:33 pak21 Exp $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -189,6 +189,9 @@ int snapshot_copy_from( libspectrum_snap *snap )
 
   tstates = libspectrum_snap_tstates( snap );
 
+  z80.interrupts_enabled_at =
+    libspectrum_snap_last_instruction_ei( snap ) ? tstates : -1;
+
   for( i=0; i<8; i++ ) {
     if( libspectrum_snap_pages( snap, i ) )
       memcpy( RAM[i], libspectrum_snap_pages( snap, i ), 0x4000 );
@@ -295,6 +298,9 @@ int snapshot_copy_to( libspectrum_snap *snap )
   libspectrum_snap_set_im( snap, IM );
 
   libspectrum_snap_set_halted( snap, z80.halted );
+  libspectrum_snap_set_last_instruction_ei(
+    snap, z80.interrupts_enabled_at == tstates
+  );
 
   libspectrum_snap_set_out_ula( snap, spectrum_last_ula );
   
