@@ -1,4 +1,4 @@
-/* snapshot.h: snapshot handling routines
+#/* snapshot.h: snapshot handling routines
    Copyright (c) 1999,2001-2002 Philip Kendall
 
    This program is free software; you can redistribute it and/or modify
@@ -25,8 +25,19 @@
 #ifndef FUSE_SNAPSHOT_H
 #define FUSE_SNAPSHOT_H
 
+#ifndef LIBSPECTRUM_LIBSPECTRUM_H
+#include <libspectrum.h>
+#endif				/* #ifndef LIBSPECTRUM_LIBSPECTRUM_H */
+
 int snapshot_read( const char *filename );
+int snapshot_read_buffer( const unsigned char *buffer, size_t length,
+			  libspectrum_id_t type );
+
+int snapshot_copy_from( libspectrum_snap *snap );
+
 int snapshot_write( const char *filename );
+int snapshot_copy_to( libspectrum_snap *snap );
+
 void snapshot_flush_slt( void );
 
 #endif
