@@ -35,27 +35,20 @@
 
 #include <config.h>
 
-#include <stdio.h>
-#include <string.h>
-#include <stdlib.h>
-#include <errno.h>
-#include <unistd.h>
-#include <sys/types.h>
-#include <sys/ioctl.h>
-#include <fcntl.h>
-
 #include <libspectrum.h>
 
 #include "fuse.h"
 #include "machine.h"
-#include "osssound.h"
-#include "sdlsound.h"
 #include "settings.h"
 #include "sound.h"
-#include "spectrum.h"
-#include "sunsound.h"
 #include "ui/ui.h"
 
+/* Do we have any of our sound devices available? */
+
+#if defined(HAVE_SYS_SOUNDCARD_H) || defined(HAVE_SYS_AUDIOIO_H) || defined(UI_SDL) || defined(HAVE_DSOUND_H)
+#define HAVE_SOUND
+#endif
+
 /* configuration */
 int sound_enabled=0;		/* Are we currently using the sound card;
 				   cf fuse.c:fuse_sound_in_use */
@@ -167,7 +160,7 @@ static int first_init=1;
 int f,ret;
 
 /* if we don't have any sound I/O code compiled in, don't do sound */
-#if !defined(HAVE_SYS_SOUNDCARD_H) && !defined(HAVE_SYS_AUDIOIO_H) && !defined(UI_SDL)
+#ifndef HAVE_SOUND
 return;
 #endif
 
@@ -185,14 +178,7 @@ sound_stereo_beeper=settings_current.stereo_beeper;
 /* only try for stereo if we need it */
 if(sound_stereo_ay || sound_stereo_beeper)
   sound_stereo=1;
-
-#if defined(UI_SDL)
-ret=sdlsound_init(device,&sound_freq,&sound_stereo);
-#elif defined(HAVE_SYS_SOUNDCARD_H)
-ret=osssound_init(device,&sound_freq,&sound_stereo);
-#elif defined(HAVE_SYS_AUDIOIO_H)
-ret=sunsound_init(device,&sound_freq,&sound_stereo);
-#endif
+ret=sound_lowlevel_init(device,&sound_freq,&sound_stereo);
 
 if(ret)
   return;
@@ -279,13 +265,7 @@ if(sound_enabled)
   {
   if(sound_buf)
     free(sound_buf);
-#if defined(UI_SDL)
-  sdlsound_end();
-#elif defined(HAVE_SYS_SOUNDCARD_H)
-  osssound_end();
-#elif defined(HAVE_SYS_AUDIOIO_H)
-  sunsound_end();
-#endif
+  sound_lowlevel_end();
   sound_enabled=0;
   }
 }
@@ -720,13 +700,7 @@ for(f=0;f<sound_framesiz;f++,tptr++)
 /* overlay AY sound */
 sound_ay_overlay();
 
-#if defined(UI_SDL)
-sdlsound_frame(sound_buf,sound_framesiz*sound_channels);
-#elif defined(HAVE_SYS_SOUNDCARD_H)
-osssound_frame(sound_buf,sound_framesiz*sound_channels);
-#elif defined(HAVE_SYS_AUDIOIO_H)
-sunsound_frame(sound_buf,sound_framesiz*sound_channels);
-#endif
+sound_lowlevel_frame(sound_buf,sound_framesiz*sound_channels);
 
 sound_oldpos[0]=sound_oldpos[1]=-1;
 sound_fillpos[0]=sound_fillpos[1]=0;
@@ -808,3 +782,25 @@ sound_oldpos[bchan]=newpos;
 sound_fillpos[bchan]=newpos+1;
 sound_oldval[bchan]=sound_oldval_orig[bchan]=val;
 }
+
+#ifndef HAVE_SOUND
+
+/* Dummy functions for when we don't have a sound device; should never be
+   called, so just abort if they are */
+int sound_lowlevel_init(const char *device,int *freqptr,int *stereoptr)
+{
+fuse_abort();
+return 0;
+}
+
+void sound_lowlevel_end( void )
+{
+fuse_abort();
+}
+
+void sound_lowlevel_frame(unsigned char *data,int len)
+{
+fuse_abort();
+}
+
+#endif
