@@ -1,5 +1,7 @@
 /* sound.h: Sound support
-   Copyright (c) 2000-2002 Russell Marks, Matan Ziv-Av, Philip Kendall
+   Copyright (c) 2000-2004 Russell Marks, Matan Ziv-Av, Philip Kendall
+
+   $Id: sound.h,v 1.17.2.1 2004/02/11 21:50:03 pak21 Exp $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -35,6 +37,7 @@ void sound_frame( void );
 void sound_beeper( int is_tape, int on );
 
 extern int sound_enabled;
+extern int sound_enabled_ever;
 extern int sound_freq;
 extern int sound_stereo;
 extern int sound_stereo_beeper;
@@ -42,4 +45,10 @@ extern int sound_stereo_ay;
 extern int sound_stereo_ay_abc;
 extern int sound_stereo_ay_narrow;
 
+/* The low-level sound interface */
+
+int sound_lowlevel_init( const char *device, int *freqptr, int *stereoptr );
+void sound_lowlevel_end( void );
+void sound_lowlevel_frame( unsigned char *data, int len );
+
 #endif				/* #ifndef FUSE_SOUND_H */
