@@ -1,7 +1,7 @@
 /* spec128.c: Spectrum 128K specific routines
    Copyright (c) 1999-2003 Philip Kendall
 
-   $Id: spec128.c,v 1.46 2003/07/11 11:03:25 pak21 Exp $
+   $Id: spec128.c,v 1.51 2003/12/27 12:40:06 pak21 Exp $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -32,15 +32,13 @@
 
 #include "ay.h"
 #include "compat.h"
-#include "display.h"
 #include "joystick.h"
 #include "machine.h"
+#include "memory.h"
 #include "settings.h"
 #include "spec128.h"
 #include "spectrum.h"
 
-static libspectrum_dword spec128_contend_delay( void );
-
 spectrum_port_info spec128_peripherals[] = {
   { 0x0001, 0x0000, spectrum_ula_read, spectrum_ula_write },
   { 0x00e0, 0x0000, joystick_kempston_read, spectrum_port_nowrite },
@@ -59,20 +57,7 @@ spec128_unattached_port( void )
 libspectrum_byte
 spec128_read_screen_memory( libspectrum_word offset )
 {
-  return RAM[machine_current->ram.current_screen][offset];
-}
-
-libspectrum_dword
-spec128_contend_memory( libspectrum_word address )
-{
-  /* Contention occurs in pages 1,3, 5 and 7. 0x4000 to 0x7fff is always page
-     5, whilst 0xc000 to 0xffff could have one of the contended pages in */
-  if( ( address >= 0x4000 && address < 0x8000 ) ||
-      ( address >= 0xc000 && ( machine_current->ram.current_page & 0x01 ) )
-    )
-    return spec128_contend_delay();
-
-  return 0;
+  return RAM[ memory_current_screen ][offset];
 }
 
 libspectrum_dword
@@ -80,28 +65,28 @@ spec128_contend_port( libspectrum_word port )
 {
   /* Contention occurs for the ULA, or for the memory paging port */
   if( ( port & 0x0001 ) == 0x0000 ||
-      ( port & 0x8002 ) == 0x0000    ) return spec128_contend_delay();
+      ( port & 0x8002 ) == 0x0000    ) return spec128_contend_delay( tstates );
 
   return 0;
 }
 
-static libspectrum_dword
-spec128_contend_delay( void )
+libspectrum_byte
+spec128_contend_delay( libspectrum_dword time )
 {
   libspectrum_word tstates_through_line;
   
   /* No contention in the upper border */
-  if( tstates < machine_current->line_times[ DISPLAY_BORDER_HEIGHT ] )
+  if( time < machine_current->line_times[ DISPLAY_BORDER_HEIGHT ] )
     return 0;
 
   /* Or the lower border */
-  if( tstates >= machine_current->line_times[ DISPLAY_BORDER_HEIGHT + 
-                                              DISPLAY_HEIGHT          ] )
+  if( time >= machine_current->line_times[ DISPLAY_BORDER_HEIGHT + 
+					   DISPLAY_HEIGHT          ] )
     return 0;
 
   /* Work out where we are in this line */
   tstates_through_line =
-    ( tstates + machine_current->timings.left_border ) %
+    ( time + machine_current->timings.left_border ) %
     machine_current->timings.tstates_per_line;
 
   /* No contention if we're in the left border */
@@ -141,13 +126,9 @@ int spec128_init( fuse_machine_info *machine )
   error = machine_set_timings( machine ); if( error ) return error;
 
   machine->timex = 0;
-  machine->ram.read_memory	     = spec128_readbyte;
-  machine->ram.read_memory_internal  = spec128_readbyte_internal;
   machine->ram.read_screen	     = spec128_read_screen_memory;
-  machine->ram.write_memory          = spec128_writebyte;
-  machine->ram.write_memory_internal = spec128_writebyte_internal;
-  machine->ram.contend_memory	     = spec128_contend_memory;
   machine->ram.contend_port	     = spec128_contend_port;
+  machine->ram.contend_delay	     = spec128_contend_delay;
 
   error = machine_allocate_roms( machine, 2 );
   if( error ) return error;
@@ -168,11 +149,6 @@ int spec128_reset(void)
 {
   int error;
 
-  machine_current->ram.locked=0;
-  machine_current->ram.current_page=0;
-  machine_current->ram.current_rom=0;
-  machine_current->ram.current_screen=5;
-
   error = machine_load_rom( &ROM[0], settings_current.rom_128_0,
 			    machine_current->rom_length[0] );
   if( error ) return error;
@@ -180,6 +156,48 @@ int spec128_reset(void)
 			    machine_current->rom_length[1] );
   if( error ) return error;
 
+  return spec128_common_reset( 1 );
+}
+
+int
+spec128_common_reset( int contention )
+{
+  size_t i;
+
+  machine_current->ram.locked=0;
+  machine_current->ram.current_page=0;
+  machine_current->ram.current_rom=0;
+
+  memory_map[0].page = &ROM[0][0x0000];
+  memory_map[1].page = &ROM[0][0x2000];
+  memory_map[0].reverse = memory_map[1].reverse = -1;
+
+  memory_map[2].page = &RAM[5][0x0000];
+  memory_map[3].page = &RAM[5][0x2000];
+  memory_map[2].reverse = memory_map[3].reverse = 5;
+
+  memory_map[4].page = &RAM[2][0x0000];
+  memory_map[5].page = &RAM[2][0x2000];
+  memory_map[4].reverse = memory_map[5].reverse = 2;
+
+  memory_map[6].page = &RAM[0][0x0000];
+  memory_map[7].page = &RAM[0][0x2000];
+  memory_map[6].reverse = memory_map[7].reverse = 0;
+
+  for( i = 0; i < 8; i++ ) memory_map[i].offset = ( i & 1 ? 0x2000 : 0x0000 );
+
+  memory_map[0].writable = memory_map[1].writable = 0;
+  for( i = 2; i < 8; i++ ) memory_map[i].writable = 1;
+
+  for( i = 0; i < 8; i++ ) memory_map[i].contended = 0;
+
+  if( contention ) {
+    memory_map[2].contended = memory_map[3].contended = 1;
+  }
+
+  memory_current_screen = 5;
+  memory_screen_mask = 0xffff;
+
   return 0;
 }
 
@@ -187,26 +205,34 @@ void
 spec128_memoryport_write( libspectrum_word port GCC_UNUSED,
 			  libspectrum_byte b )
 {
-  int old_screen;
+  int page, rom, screen;
 
-  /* Do nothing if we've locked the RAM configuration */
-  if(machine_current->ram.locked) return;
+  if( machine_current->ram.locked ) return;
     
-  old_screen=machine_current->ram.current_screen;
+  page = b & 0x07;
+  screen = ( b & 0x08 ) ? 7 : 5;
+  rom = ( b & 0x10 ) >> 4;
 
-  /* Store the last byte written in case we need it */
-  machine_current->ram.last_byte=b;
+  memory_map[0].page = &ROM[ rom ][0x0000];
+  memory_map[1].page = &ROM[ rom ][0x2000];
 
-  /* Work out which page, screen and ROM are selected */
-  machine_current->ram.current_page= b & 0x07;
-  machine_current->ram.current_screen=( b & 0x08 ) ? 7 : 5;
-  machine_current->ram.current_rom=(machine_current->ram.current_rom & 0x02) |
-    ( (b & 0x10) >> 4 );
+  memory_map[6].page = &RAM[ page ][0x0000];
+  memory_map[7].page = &RAM[ page ][0x2000];
+  memory_map[6].reverse = memory_map[7].reverse = page;
 
-  /* Are we locking the RAM configuration? */
-  machine_current->ram.locked=( b & 0x20 );
+  /* Pages 1, 3, 5 and 7 are contended */
+  memory_map[6].contended = memory_map[7].contended = page & 0x01;
 
   /* If we changed the active screen, mark the entire display file as
      dirty so we redraw it on the next pass */
-  if(machine_current->ram.current_screen!=old_screen) display_refresh_all();
+  if( memory_current_screen != screen ) {
+    display_refresh_all();
+    memory_current_screen = screen;
+  }
+
+  machine_current->ram.current_page = page;
+  machine_current->ram.current_rom = rom;
+  machine_current->ram.locked = ( b & 0x20 );
+
+  machine_current->ram.last_byte = b;
 }
