@@ -1,7 +1,7 @@
 /* spec128.h: Spectrum 128K specific routines
    Copyright (c) 1999-2001 Philip Kendall
 
-   $Id: spec128.h,v 1.17 2003/07/11 11:03:25 pak21 Exp $
+   $Id: spec128.h,v 1.20 2003/11/14 22:36:52 pak21 Exp $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -46,9 +46,11 @@ void spec128_writebyte_internal( libspectrum_word address,
 
 libspectrum_dword spec128_contend_memory( libspectrum_word address );
 libspectrum_dword spec128_contend_port( libspectrum_word port );
+libspectrum_byte spec128_contend_delay( libspectrum_dword time );
 
 int spec128_init( fuse_machine_info *machine );
 int spec128_reset(void);
+int spec128_common_reset( int contention );
 
 void spec128_memoryport_write( libspectrum_word port, libspectrum_byte b );
 
