@@ -1,7 +1,7 @@
 /* spec16.h: Spectrum 16K specific routines
    Copyright (c) 1999-2003 Philip Kendall
 
-   $Id: spec16.h,v 1.2 2003/07/11 11:03:25 pak21 Exp $
+   $Id: spec16.h,v 1.3 2003/11/14 11:46:46 pak21 Exp $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -29,6 +29,10 @@
 
 #include <libspectrum.h>
 
+#ifndef FUSE_MACHINE_H
+#include "machine.h"
+#endif			/* #ifndef FUSE_MACHINE_H */
+
 libspectrum_byte spec16_readbyte( libspectrum_word address );
 libspectrum_byte spec16_readbyte_internal( libspectrum_word address );
 void spec16_writebyte( libspectrum_word address, libspectrum_byte b );
