@@ -1,7 +1,7 @@
 /* specplus2.c: Spectrum +2 specific routines
    Copyright (c) 1999-2001 Philip Kendall
 
-   $Id: specplus2.c,v 1.30 2003/02/07 15:07:13 pak21 Exp $
+   $Id: specplus2.c,v 1.32 2003/11/14 21:03:55 pak21 Exp $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -48,13 +48,9 @@ int specplus2_init( fuse_machine_info *machine )
   error = machine_set_timings( machine ); if( error ) return error;
 
   machine->timex = 0;
-  machine->ram.read_memory	     = spec128_readbyte;
-  machine->ram.read_memory_internal  = spec128_readbyte_internal;
   machine->ram.read_screen	     = spec128_read_screen_memory;
-  machine->ram.write_memory          = spec128_writebyte;
-  machine->ram.write_memory_internal = spec128_writebyte_internal;
-  machine->ram.contend_memory	     = spec128_contend_memory;
   machine->ram.contend_port	     = spec128_contend_port;
+  machine->ram.contend_delay	     = spec128_contend_delay;
 
   error = machine_allocate_roms( machine, 2 );
   if( error ) return error;
@@ -76,11 +72,6 @@ specplus2_reset( void )
 {
   int error;
 
-  machine_current->ram.locked=0;
-  machine_current->ram.current_page=0;
-  machine_current->ram.current_rom=0;
-  machine_current->ram.current_screen=5;
-
   error = machine_load_rom( &ROM[0], settings_current.rom_plus2_0,
 			    machine_current->rom_length[0] );
   if( error ) return error;
@@ -88,5 +79,5 @@ specplus2_reset( void )
 			    machine_current->rom_length[1] );
   if( error ) return error;
 
-  return 0;
+  return spec128_common_reset( 1 );
 }
