@@ -1,7 +1,7 @@
 /* specplus3.h: Spectrum +2A/+3 specific routines
    Copyright (c) 1999-2003 Philip Kendall
 
-   $Id: specplus3.h,v 1.24 2003/07/11 11:03:25 pak21 Exp $
+   $Id: specplus3.h,v 1.29 2004/01/05 23:10:18 pak21 Exp $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -51,11 +51,15 @@ void specplus3_writebyte_internal( libspectrum_word address,
 
 libspectrum_dword specplus3_contend_memory( libspectrum_word address );
 libspectrum_dword specplus3_contend_port( libspectrum_word address );
+libspectrum_byte specplus3_contend_delay( libspectrum_dword time );
 
 int specplus3_init( fuse_machine_info *machine );
 int specplus3_reset(void);
 
+int specplus3_plus2a_common_reset( void );
+
 void specplus3_memoryport_write( libspectrum_word port, libspectrum_byte b );
+void specplus3_memoryport2_write( libspectrum_word port, libspectrum_byte b );
 
 /* We need these outside the HAVE_765_H guards as they're also used
    for identifying the TRDOS drives */
@@ -68,19 +72,19 @@ typedef enum specplus3_drive_number {
 /* The +3's drives */
 
 typedef struct specplus3_drive_t {
-  int fd;			/* Our file descriptor for this disk; note
-				   that lib765 will use a stdio stream
-				   as well internally; if -1 => no disk
-				   in drive */
-  dev_t device;			/* The device and inode where this disk file */
-  ino_t inode;			/* resides. Used to check for the same file
-				   being put into both drives */
+  int fd;			/* The file descriptor for the
+				   temporary file we are using for
+				   this disk */
+  char filename[ PATH_MAX ];	/* And the name of the temporary file */
+
   FDRV_PTR drive;		/* The lib765 structure for this drive */
 } specplus3_drive_t;
 
+int specplus3_disk_present( specplus3_drive_number which );
 int specplus3_disk_insert( specplus3_drive_number which,
 			   const char *filename );
-int specplus3_disk_eject( specplus3_drive_number which );
+int specplus3_disk_eject( specplus3_drive_number which, int save );
+int specplus3_disk_write( specplus3_drive_number which, const char *filename );
 #endif			/* #ifdef HAVE_765_H */
 
 #endif			/* #ifndef FUSE_SPECPLUS3_H */
