@@ -1,7 +1,7 @@
 /* spectrum.c: Generic Spectrum routines
    Copyright (c) 1999-2003 Philip Kendall, Darren Salt
 
-   $Id: spectrum.c,v 1.39 2003/07/11 11:03:25 pak21 Exp $
+   $Id: spectrum.c,v 1.48 2003/12/14 12:34:36 pak21 Exp $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -66,14 +66,12 @@ libspectrum_byte spectrum_last_ula;
 
 /* Set these every time we change machine to avoid having to do a
    structure lookup too often */
-spectrum_memory_read_function readbyte;
-spectrum_memory_read_function readbyte_internal;
 spectrum_screen_read_function read_screen_memory;
-spectrum_memory_write_function writebyte;
-spectrum_memory_write_function writebyte_internal;
 
-spectrum_memory_contention_function contend_memory;
 spectrum_port_contention_function contend_port;
+spectrum_contention_delay_function contend_delay;
+
+libspectrum_byte spectrum_contention[ 80000 ];
 
 /* Level data from .slt files */
 
@@ -84,46 +82,41 @@ libspectrum_byte *slt_screen;	/* The screenshot from the .slt file */
 int slt_screen_level;		/* The level of the screenshot.
 				   Not used for anything AFAIK */
 
-int spectrum_interrupt(void)
+int
+spectrum_frame( void )
 {
-  /* FIXME: find some better way to deal with speed == 0 */
-  int speed =
-    settings_current.emulation_speed < 1 ? 1 :
-    settings_current.emulation_speed;
+  libspectrum_dword frame_length;
+  int error;
 
   /* Reduce the t-state count of both the processor and all the events
      scheduled to occur. Done slightly differently if RZX playback is
-     occuring
-  */
-  if( rzx_playback ) {
-    if( event_interrupt( tstates ) ) return 1;
-    tstates = 0;
-  } else {
-    if(event_interrupt( machine_current->timings.tstates_per_frame )) return 1;
-    tstates -= machine_current->timings.tstates_per_frame;
-  }
+     occuring */
+  frame_length = rzx_playback ? tstates
+			      : machine_current->timings.tstates_per_frame;
 
-#ifdef UI_SDL		/* SDL sound routines do not provide speed control */
+  error = event_frame( frame_length ); if( error ) return error;
+  tstates -= frame_length;
+  if( z80.interrupts_enabled_at >= 0 )
+    z80.interrupts_enabled_at -= frame_length;
+
+#ifdef ALWAYS_USE_TIMER /* Some sound routines do not provide speed control */
   if( sound_enabled ) sound_frame();
   timer_sleep();
-  timer_count -= 100.0 / speed;
-#else			/* #ifdef UI_SDL */
+#else			/* #ifdef ALWAYS_USE_TIMER */
   if(sound_enabled) {
     sound_frame();
   } else {
     timer_sleep();
-    timer_count -= 100.0 / speed;
   }
-#endif			/* #ifdef UI_SDL */
+#endif			/* #ifdef ALWAYS_USE_TIMER */
 
   if(display_frame()) return 1;
   printer_frame();
-  z80_interrupt();
 
   /* Add an interrupt unless they're being generated by .rzx playback */
-  if( ! rzx_playback ) {
-    if(event_add(machine_current->timings.tstates_per_frame,
-		 EVENT_TYPE_INTERRUPT) ) return 1;
+  if( !rzx_playback ) {
+    if( event_add( machine_current->timings.tstates_per_frame,
+		   EVENT_TYPE_FRAME ) ) return 1;
   }
 
   return 0;
