@@ -1,7 +1,7 @@
 /* spectrum.h: Spectrum 48K specific routines
    Copyright (c) 1999-2003 Philip Kendall, Darren Salt
 
-   $Id: spectrum.h,v 1.27 2003/07/11 11:03:25 pak21 Exp $
+   $Id: spectrum.h,v 1.32 2003/12/27 12:40:06 pak21 Exp $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -50,56 +50,43 @@ extern libspectrum_byte **ROM;
 /* And the RAM */
 extern libspectrum_byte RAM[8][0x4000];
 
-typedef libspectrum_byte
-  (*spectrum_memory_read_function)( libspectrum_word address );
 typedef libspectrum_byte
   (*spectrum_screen_read_function)( libspectrum_word offset );
-typedef void (*spectrum_memory_write_function)( libspectrum_word address,
-						libspectrum_byte data );
-typedef libspectrum_dword
-  (*spectrum_memory_contention_function)( libspectrum_word address );
+
 typedef libspectrum_dword
   (*spectrum_port_contention_function)( libspectrum_word port );
+typedef libspectrum_byte
+  (*spectrum_contention_delay_function)( libspectrum_dword time );
 
 typedef struct spectrum_raminfo {
 
-  spectrum_memory_read_function read_memory; /* Read a byte from anywhere in 
-						paged in memory */
-  spectrum_memory_read_function read_memory_internal;
-
   spectrum_screen_read_function read_screen; /* Read a byte from the
 						current screen */
 
-  spectrum_memory_write_function write_memory; /* Write to paged-in memory */
-  spectrum_memory_write_function write_memory_internal;
-
-  spectrum_memory_contention_function contend_memory; /* How long must we wait
-							 to access memory? */
   spectrum_port_contention_function contend_port; /* And how long to access
 						     an IO port? */
 
+  /* What's the actual delay at the current tstate */
+  spectrum_contention_delay_function contend_delay;
+
   int locked;			/* Is the memory configuration locked? */
-  int current_page,current_rom,current_screen; /* Current paged memory */
+  int current_page, current_rom; /* Current paged memory */
 
   libspectrum_byte last_byte;	/* The last byte sent to the 128K port */
   libspectrum_byte last_byte2;	/* The last byte sent to +3 port */
 
   int special;			/* Is a +3 special config in use? */
-  int specialcfg;		/* If so, which one? */
 
 } spectrum_raminfo;
 
 /* Set these every time we change machine to avoid having to do a
    structure lookup too often */
-extern spectrum_memory_read_function readbyte;
-extern spectrum_memory_read_function readbyte_internal;
 extern spectrum_screen_read_function read_screen_memory;
-extern spectrum_memory_write_function writebyte;
-extern spectrum_memory_write_function writebyte_internal;
-
-extern spectrum_memory_contention_function contend_memory;
 extern spectrum_port_contention_function contend_port;
 
+/* How much contention do we get at every tstate? */
+extern libspectrum_byte spectrum_contention[ 80000 ];
+
 /* Things relating to peripherals */
 
 typedef libspectrum_byte
@@ -127,7 +114,7 @@ libspectrum_byte spectrum_unattached_port( int offset );
 
 /* Miscellaneous stuff */
 
-int spectrum_interrupt(void);
+int spectrum_frame( void );
 
 /* Data from .slt files */
 
