@@ -1,7 +1,7 @@
 /* sunsound.c: OpenBSD sound I/O
-   Copyright (c) 2002 Alexander Yurchenko, Russell Marks, Philip Kendall
+   Copyright (c) 2002-2003 Alexander Yurchenko, Russell Marks, Philip Kendall
 
-   $Id: sunsound.c,v 1.9 2002/12/11 15:29:23 pak21 Exp $
+   $Id: sunsound.c,v 1.11 2003/12/14 23:16:39 pak21 Exp $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -40,7 +40,8 @@
 #endif
 #include <unistd.h>
 
-#include "sunsound.h"
+#include "settings.h"
+#include "sound.h"
 #include "ui/ui.h"
 
 /* using (8) 64 byte frags for 8kHz, scale up for higher */
@@ -50,7 +51,7 @@ static int soundfd = -1;
 static int sixteenbit = 0;
 
 int
-sunsound_init(const char *device,int *freqptr, int *stereoptr)
+sound_lowlevel_init( const char *device, int *freqptr, int *stereoptr )
 {
 #ifndef solaris
 	int frag;
@@ -65,11 +66,13 @@ sunsound_init(const char *device,int *freqptr, int *stereoptr)
 	   being used by something else, but then set it blocking
 	   again as that's what we actually want */
 	if ((soundfd = open(device, O_WRONLY | O_NONBLOCK )) == -1) {
+		settings_current.sound = 0;
 	        ui_error( UI_ERROR_ERROR, "Couldn't open sound device '%s'",
 			  device );
 		return 1;
 	}
 	if ((flags = fcntl(soundfd, F_GETFL)) == -1) {
+		settings_current.sound = 0;
 	        ui_error( UI_ERROR_ERROR, "Couldn't fcntl sound device '%s'",
 			  device );
 	        close(soundfd);
@@ -77,6 +80,7 @@ sunsound_init(const char *device,int *freqptr, int *stereoptr)
 	}
 	flags &= ~O_NONBLOCK;
 	if (fcntl(soundfd, F_SETFL, flags) == -1) {
+		settings_current.sound = 0;
 	        ui_error( UI_ERROR_ERROR,
 			  "Couldn't set sound device '%s' blocking",
 			  device );
@@ -93,6 +97,7 @@ sunsound_init(const char *device,int *freqptr, int *stereoptr)
 		ai.play.encoding = AUDIO_ENCODING_LINEAR;
 		ai.play.precision = 16;
 		if (ioctl(soundfd, AUDIO_SETINFO, &ai) == -1) {
+			settings_current.sound = 0;
 		        ui_error( UI_ERROR_ERROR,
 				"Couldn't set bit size of sound device '%s'",
 				  device );
@@ -107,17 +112,20 @@ sunsound_init(const char *device,int *freqptr, int *stereoptr)
 		/* if it failed make sure the opposite is ok */
 		ai.play.channels = *stereoptr ? 1 : 2;
 		if (ioctl(soundfd, AUDIO_SETINFO, &ai) == -1) {
+			settings_current.sound = 0;
 		        ui_error( UI_ERROR_ERROR,
 				"Couldn't set channels of sound device '%s'",
 				  device );
 			close(soundfd);
 			return 1;
 		}
+		/* FIXME: is this line correct? */
 		*stereoptr = *stereoptr ? 1 : 2;
 	}
 
 	ai.play.sample_rate = *freqptr;
 	if (ioctl(soundfd, AUDIO_SETINFO, &ai) == -1) {
+		settings_current.sound = 0;
 	        ui_error( UI_ERROR_ERROR,
 			  "Couldn't set sample rate of sound device '%s'",
 			  device );
@@ -142,6 +150,7 @@ sunsound_init(const char *device,int *freqptr, int *stereoptr)
 	if (ai.hiwat == 0)
 		ai.hiwat = 65536;
 	if (ioctl(soundfd, AUDIO_SETINFO, &ai) == -1) {
+		settings_current.sound = 0;
 	        ui_error( UI_ERROR_ERROR,
 			  "Couldn't set block size of sound device '%s'",
 			  device );
@@ -154,7 +163,7 @@ sunsound_init(const char *device,int *freqptr, int *stereoptr)
 }
 
 void
-sunsound_end()
+sound_lowlevel_end( void )
 {
 #ifdef solaris
 	ioctl(soundfd, I_FLUSH, FLUSHW);
@@ -165,7 +174,7 @@ sunsound_end()
 }
 
 void
-sunsound_frame(data, len)
+sound_lowlevel_frame(data, len)
 	unsigned char *data;
 	int len;
 {
