@@ -1,7 +1,7 @@
 /* tape.c: tape handling routines
-   Copyright (c) 1999-2003 Philip Kendall, Darren Salt, Witold Filipczyk
+   Copyright (c) 1999-2004 Philip Kendall, Darren Salt, Witold Filipczyk
 
-   $Id: tape.c,v 1.93 2003/07/19 09:17:44 pak21 Exp $
+   $Id: tape.c,v 1.106 2004/01/06 12:33:12 pak21 Exp $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -36,10 +36,10 @@
 #include "event.h"
 #include "fuse.h"
 #include "machine.h"
+#include "memory.h"
 #include "scld.h"
 #include "settings.h"
 #include "sound.h"
-#include "spectrum.h"
 #include "settings.h"
 #include "snapshot.h"
 #include "tape.h"
@@ -50,10 +50,13 @@
 #include "z80/z80_macros.h"
 
 /* The current tape */
-libspectrum_tape *tape;
+static libspectrum_tape *tape;
+
+/* Has the current tape been modified since it was last loaded/saved? */
+int tape_modified;
 
 /* Is the emulated tape deck playing? */
-int tape_playing;
+static int tape_playing;
 
 /* Is there a high input to the EAR socket? */
 int tape_microphone;
@@ -63,6 +66,7 @@ int tape_microphone;
 static int tape_autoload( libspectrum_machine hardware );
 static int trap_load_block( libspectrum_tape_block *block );
 int trap_check_rom( void );
+static void make_name( char *name, const char *data );
 
 /* Function defintions */
 
@@ -73,6 +77,10 @@ int tape_init( void )
   error = libspectrum_tape_alloc( &tape );
   if( !tape ) return error;
 
+  tape_modified = 0;
+
+  /* Don't call tape_stop() here as the UI hasn't been initialised yet,
+     so we can't update the statusbar */
   tape_playing = 0;
   tape_microphone = 0;
   if( settings_current.sound_load ) sound_beeper( 1, tape_microphone );
@@ -91,32 +99,14 @@ int tape_open( const char *filename, int autoload )
   utils_file file;
   int error;
 
-  /* If we already have a tape file open, close it */
-  if( libspectrum_tape_present( tape ) ) {
-    error = tape_close();
-    if( error ) { utils_close_file( &file ); return error; }
-  }
-
-  /* Get the file's data */
   error = utils_read_file( filename, &file );
   if( error ) return error;
 
-  error = libspectrum_tape_read( tape, file.buffer, file.length,
-				 LIBSPECTRUM_ID_UNKNOWN, filename );
-  if( error != LIBSPECTRUM_ERROR_NONE ) {
-    utils_close_file( &file );
-    return error;
-  }
+  error = tape_read_buffer( file.buffer, file.length, LIBSPECTRUM_ID_UNKNOWN,
+			    filename, autoload );
+  if( error ) { utils_close_file( &file ); return error; }
 
-  if( utils_close_file( &file ) ) {
-    tape_close();
-    return 1;
-  }
-
-  if( autoload ) {
-    error = tape_autoload( machine_current->machine );
-    if( error ) return error;
-  }
+  utils_close_file( &file );
 
   return 0;
 }
@@ -124,7 +114,7 @@ int tape_open( const char *filename, int autoload )
 /* Use an already open tape file as the current tape */
 int
 tape_read_buffer( unsigned char *buffer, size_t length, libspectrum_id_t type,
-		  int autoload )
+		  const char *filename, int autoload )
 {
   int error;
 
@@ -132,9 +122,12 @@ tape_read_buffer( unsigned char *buffer, size_t length, libspectrum_id_t type,
     error = tape_close(); if( error ) return error;
   }
 
-  error = libspectrum_tape_read( tape, buffer, length, type, NULL );
+  error = libspectrum_tape_read( tape, buffer, length, type, filename );
   if( error ) return error;
 
+  tape_modified = 0;
+  ui_tape_browser_update();
+
   if( autoload ) {
     error = tape_autoload( machine_current->machine );
     if( error ) return error;
@@ -158,7 +151,7 @@ tape_autoload( libspectrum_machine hardware )
   }
 
   snprintf( filename, 80, "tape_%s.z80", id );
-  fd = utils_find_lib( filename );
+  fd = utils_find_auxiliary_file( filename, UTILS_AUXILIARY_LIB );
   if( fd == -1 ) {
     ui_error( UI_ERROR_ERROR,
 	      "Couldn't find autoload snap for machine type '%s'", id );
@@ -185,6 +178,23 @@ tape_autoload( libspectrum_machine hardware )
 int tape_close( void )
 {
   int error;
+  ui_confirm_save_t confirm;
+
+  /* If the tape has been modified, check if we want to do this */
+  if( tape_modified ) {
+
+    confirm = ui_confirm_save( "Tape has been modified" );
+    switch( confirm ) {
+
+    case UI_CONFIRM_SAVE_SAVE:
+      error = ui_tape_write(); if( error ) return error;
+      break;
+
+    case UI_CONFIRM_SAVE_DONTSAVE: break;
+    case UI_CONFIRM_SAVE_CANCEL: return 1;
+
+    }
+  }
 
   /* Stop the tape if it's currently playing */
   if( tape_playing ) {
@@ -196,12 +206,28 @@ int tape_close( void )
   error = libspectrum_tape_clear( tape );
   if( error ) return error;
 
+  tape_modified = 0;
+  ui_tape_browser_update();
+
   return 0;
 }
 
 /* Select the nth block on the tape; 0 => 1st block */
 int
 tape_select_block( size_t n )
+{
+  int error;
+
+  error = tape_select_block_no_update( n ); if( error ) return error;
+
+  ui_tape_browser_update();
+
+  return 0;
+}
+
+/* The same, but without updating the browser display */
+int
+tape_select_block_no_update( size_t n )
 {
   return libspectrum_tape_nth_block( tape, n );
 }
@@ -235,6 +261,9 @@ int tape_write( const char* filename )
   error = utils_write_file( filename, buffer, length );
   if( error ) { free( buffer ); return error; }
 
+  tape_modified = 0;
+  ui_tape_browser_update();
+
   return 0;
 
 }
@@ -285,9 +314,12 @@ int tape_load_trap( void )
   next_block = libspectrum_tape_peek_next_block( tape );
 
   if( libspectrum_tape_block_type(next_block) == LIBSPECTRUM_TAPE_BLOCK_ROM ) {
+
     next_block = libspectrum_tape_select_next_block( tape );
     if( !next_block ) return 1;
 
+    ui_tape_browser_update();
+
     return 0;
   }
 
@@ -406,6 +438,9 @@ int tape_save_trap( void )
   error = libspectrum_tape_append_block( tape, block );
   if( error ) return error;
 
+  tape_modified = 1;
+  ui_tape_browser_update();
+
   /* And then return via the RET at #053E, except on Timex 2068 at #00E4 */
   if ( machine_current->machine == LIBSPECTRUM_MACHINE_TC2068 ) {
     PC = 0x00e4;
@@ -428,7 +463,7 @@ int trap_check_rom( void )
 
   case LIBSPECTRUM_MACHINE_TC2068:
     /* OK if we're in the EXROM (location of the tape routines) */
-    return( timex_memory[0].page == timex_exrom[0].page );
+    return( memory_map[0].page == timex_exrom[0].page );
 
   case LIBSPECTRUM_MACHINE_128:
   case LIBSPECTRUM_MACHINE_PLUS2:
@@ -481,6 +516,9 @@ int tape_play( void )
   tape_playing = 1;
   tape_microphone = 0;
 
+  /* Update the status bar */
+  ui_statusbar_update( UI_STATUSBAR_ITEM_TAPE, UI_STATUSBAR_STATE_ACTIVE );
+
   /* Timex machines have no loading noise */
   if( ( !( machine_current->timex ) ) && settings_current.sound_load )
     sound_beeper( 1, tape_microphone );
@@ -502,6 +540,7 @@ int tape_toggle_play( void )
 int tape_stop( void )
 {
   tape_playing = 0;
+  ui_statusbar_update( UI_STATUSBAR_ITEM_TAPE, UI_STATUSBAR_STATE_INACTIVE );
   return 0;
 }
 
@@ -545,16 +584,20 @@ tape_next_edge( libspectrum_dword last_tstates )
     return 0;
   }
 
-  /* If that was the end of a block, tape traps are active _and_ the
-     new block is a ROM loader, return without putting another event
-     into the queue */
-  block = libspectrum_tape_current_block( tape );
-  if( ( flags & LIBSPECTRUM_TAPE_FLAGS_BLOCK ) &&
-      settings_current.tape_traps &&
-      libspectrum_tape_block_type( block ) == LIBSPECTRUM_TAPE_BLOCK_ROM
-    ) {
-    tape_playing = 0;
-    return 0;
+  /* If that was the end of a block, update the browser */
+  if( flags & LIBSPECTRUM_TAPE_FLAGS_BLOCK ) {
+
+    ui_tape_browser_update();
+
+    /* If tape traps are active _and_ the new block is a ROM loader,
+     return without putting another event into the queue */
+    block = libspectrum_tape_current_block( tape );
+    if( settings_current.tape_traps &&
+	libspectrum_tape_block_type( block ) == LIBSPECTRUM_TAPE_BLOCK_ROM
+      ) {
+      error = tape_stop(); if( error ) return error;
+      return 0;
+    }
   }
 
   /* Otherwise, put this into the event queue; remember that this edge
@@ -567,140 +610,137 @@ tape_next_edge( libspectrum_dword last_tstates )
   return 0;
 }
 
+/* Call a user-supplied function for every block in the current tape */
 int
-tape_get_block_list( char ****list, size_t *n )
+tape_foreach( void (*function)( libspectrum_tape_block *block,
+				void *user_data),
+	      void *user_data )
 {
-  libspectrum_tape_iterator iterator;
   libspectrum_tape_block *block;
-  size_t allocated;
-  int offset;
-
-  *list = NULL; *n = 0; allocated = 0;
+  libspectrum_tape_iterator iterator;
 
   for( block = libspectrum_tape_iterator_init( &iterator, tape );
        block;
        block = libspectrum_tape_iterator_next( &iterator ) )
-  {
-
-    /* Get more space if we need it */
-    if( *n == allocated ) {
-      size_t new_size = allocated ? 2 * allocated : 1;
-      char ***new_list = (char***)realloc( (*list),
-					   new_size * sizeof( char** ) );
-      if( !new_list ) {
-	tape_free_block_list( *list, *n );
-	ui_error( UI_ERROR_ERROR, "Out of memory at %s:%d",
-		  __FILE__, __LINE__ );
-	return 1;
-      }
-
-      (*list) = new_list; allocated = new_size;
-    }
-
-    (*list)[*n] = (char**)malloc( 2 * sizeof( char* ) );
-    if( !(*list)[*n] ) {
-      tape_free_block_list( *list, *n );
-      ui_error( UI_ERROR_ERROR, "Out of memory at %s:%d", __FILE__, __LINE__ );
-      return 1;
-    }
-
-    (*list)[*n][0] = malloc( 80 * sizeof( char ) );
-    if( !(*list)[*n][0] ) {
-      free( (*list)[*n] ); tape_free_block_list( *list, *n );
-      ui_error( UI_ERROR_ERROR, "Out of memory at %s:%d", __FILE__, __LINE__ );
-      return 1;
-    }
-
-    (*list)[*n][1] = malloc( 80 * sizeof( char ) );
-    if( !(*list)[*n][1] ) {
-      free( (*list)[*n][0] ); free( (*list)[*n] );
-      tape_free_block_list( *list, *n );
-      ui_error( UI_ERROR_ERROR, "Out of memory at %s:%d", __FILE__, __LINE__ );
-      return 1;
-    }
-
-    libspectrum_tape_block_description( (*list)[*n][0], 80, block );
-
-    switch( libspectrum_tape_block_type( block ) ) {
-
-    case LIBSPECTRUM_TAPE_BLOCK_ROM:
-    case LIBSPECTRUM_TAPE_BLOCK_TURBO:
-    case LIBSPECTRUM_TAPE_BLOCK_PURE_DATA:
-    case LIBSPECTRUM_TAPE_BLOCK_RAW_DATA:
-      snprintf( (*list)[*n][1], 80, "%lu bytes",
-		(unsigned long)libspectrum_tape_block_data_length( block ) );
-      break;
-
-    case LIBSPECTRUM_TAPE_BLOCK_PURE_TONE:
-      snprintf( (*list)[*n][1], 80, "%lu tstates",
-		(unsigned long)libspectrum_tape_block_pulse_length( block ) );
-      break;
-
-    case LIBSPECTRUM_TAPE_BLOCK_PULSES:
-      snprintf( (*list)[*n][1], 80, "%lu pulses",
-		(unsigned long)libspectrum_tape_block_count( block ) );
-      break;
-
-    case LIBSPECTRUM_TAPE_BLOCK_PAUSE:
-      snprintf( (*list)[*n][1], 80, "%lu ms",
-		(unsigned long)libspectrum_tape_block_pause( block ) );
-      break;
-
-    case LIBSPECTRUM_TAPE_BLOCK_GROUP_START:
-    case LIBSPECTRUM_TAPE_BLOCK_COMMENT:
-    case LIBSPECTRUM_TAPE_BLOCK_MESSAGE:
-    case LIBSPECTRUM_TAPE_BLOCK_CUSTOM:
-      snprintf( (*list)[*n][1], 80, "%s",
-		libspectrum_tape_block_text( block ) );
-      break;
-
-    case LIBSPECTRUM_TAPE_BLOCK_JUMP:
-      offset = libspectrum_tape_block_offset( block );
-      if( offset > 0 ) {
-	snprintf( (*list)[*n][1], 80, "Forward %d blocks", offset );
-      } else {
-	snprintf( (*list)[*n][1], 80, "Backward %d blocks", -offset );
-      }
-      break;
-
-    case LIBSPECTRUM_TAPE_BLOCK_LOOP_START:
-      snprintf( (*list)[*n][1], 80, "%lu iterations",
-		(unsigned long)libspectrum_tape_block_count( block ) );
-      break;
-
-    case LIBSPECTRUM_TAPE_BLOCK_SELECT:
-      snprintf( (*list)[*n][1], 80, "%lu options",
-		(unsigned long)libspectrum_tape_block_count( block ) );
-      break;
-
-    case LIBSPECTRUM_TAPE_BLOCK_GROUP_END:
-    case LIBSPECTRUM_TAPE_BLOCK_LOOP_END:
-    case LIBSPECTRUM_TAPE_BLOCK_STOP48:
-    case LIBSPECTRUM_TAPE_BLOCK_ARCHIVE_INFO:
-    case LIBSPECTRUM_TAPE_BLOCK_HARDWARE:
-    default:
-      (*list)[*n][1][0] = '\0';
-      break;
-
-    }
-
-    (*n)++;
-  }
+    function( block, user_data );
 
   return 0;
 }
 
 int
-tape_free_block_list( char ***list, size_t n )
+tape_block_details( char *buffer, size_t length,
+		    libspectrum_tape_block *block )
 {
-  size_t i;
+  libspectrum_byte *data;
+  const char *type; char name[11];
+  int offset;
+
+  switch( libspectrum_tape_block_type( block ) ) {
+
+  case LIBSPECTRUM_TAPE_BLOCK_ROM:
+    /* See if this looks like a standard Spectrum header and if so
+       display some extra data */
+    if( libspectrum_tape_block_data_length( block ) != 19 ) goto normal;
+
+    data = libspectrum_tape_block_data( block );
+
+    /* Flag byte is 0x00 for headers */
+    if( data[0] != 0x00 ) goto normal;
+
+    switch( data[1] ) {
+    case 0x00: type = "Program"; break;
+    case 0x01: type = "Number array"; break;
+    case 0x02: type = "Character array"; break;
+    case 0x03: type = "Bytes"; break;
+    default: goto normal;
+    }
+    
+    make_name( name, &data[2] );
+
+    snprintf( buffer, length, "%s: \"%s\"", type, name );
+
+    break;
+
+  normal:
+    snprintf( buffer, length, "%lu bytes",
+	      (unsigned long)libspectrum_tape_block_data_length( block ) );
+    break;
+
+  case LIBSPECTRUM_TAPE_BLOCK_TURBO:
+  case LIBSPECTRUM_TAPE_BLOCK_PURE_DATA:
+  case LIBSPECTRUM_TAPE_BLOCK_RAW_DATA:
+    snprintf( buffer, length, "%lu bytes",
+	      (unsigned long)libspectrum_tape_block_data_length( block ) );
+    break;
+
+  case LIBSPECTRUM_TAPE_BLOCK_PURE_TONE:
+    snprintf( buffer, length, "%lu tstates",
+	      (unsigned long)libspectrum_tape_block_pulse_length( block ) );
+    break;
+
+  case LIBSPECTRUM_TAPE_BLOCK_PULSES:
+    snprintf( buffer, length, "%lu pulses",
+	      (unsigned long)libspectrum_tape_block_count( block ) );
+    break;
+
+  case LIBSPECTRUM_TAPE_BLOCK_PAUSE:
+    snprintf( buffer, length, "%lu ms",
+	      (unsigned long)libspectrum_tape_block_pause( block ) );
+    break;
+
+  case LIBSPECTRUM_TAPE_BLOCK_GROUP_START:
+  case LIBSPECTRUM_TAPE_BLOCK_COMMENT:
+  case LIBSPECTRUM_TAPE_BLOCK_MESSAGE:
+  case LIBSPECTRUM_TAPE_BLOCK_CUSTOM:
+    snprintf( buffer, length, "%s", libspectrum_tape_block_text( block ) );
+    break;
+
+  case LIBSPECTRUM_TAPE_BLOCK_JUMP:
+    offset = libspectrum_tape_block_offset( block );
+    if( offset > 0 ) {
+      snprintf( buffer, length, "Forward %d blocks", offset );
+    } else {
+      snprintf( buffer, length, "Backward %d blocks", -offset );
+    }
+    break;
+
+  case LIBSPECTRUM_TAPE_BLOCK_LOOP_START:
+    snprintf( buffer, length, "%lu iterations",
+	      (unsigned long)libspectrum_tape_block_count( block ) );
+    break;
+
+  case LIBSPECTRUM_TAPE_BLOCK_SELECT:
+    snprintf( buffer, length, "%lu options",
+	      (unsigned long)libspectrum_tape_block_count( block ) );
+    break;
+
+  case LIBSPECTRUM_TAPE_BLOCK_GROUP_END:
+  case LIBSPECTRUM_TAPE_BLOCK_LOOP_END:
+  case LIBSPECTRUM_TAPE_BLOCK_STOP48:
+  case LIBSPECTRUM_TAPE_BLOCK_ARCHIVE_INFO:
+  case LIBSPECTRUM_TAPE_BLOCK_HARDWARE:
+  default:
+    buffer[0] = '\0';
+    break;
 
-  for( i=0; i<n; i++ ) {
-    free( list[i][0] ); free( list[i][1] );
-    free( list[i] );
   }
 
-  free( list );
-
   return 0;
 }
+
+static void
+make_name( char *name, const char *data )
+{
+  size_t i;
+
+  for( i = 0; i < 10; i++, name++, data++ ) {
+    if( *data >= 32 && *data < 127 ) {
+      *name = *data;
+    } else {
+      *name = '?';
+    }
+  }
+
+  *name = '\0';
+}
+
