@@ -1,7 +1,7 @@
 /* trdos.c: Routines for handling the Betadisk interface
-   Copyright (c) 2002-2003 Dmitry Sanarin, Fredrick Meunier, Philip Kendall
+   Copyright (c) 2002-2004 Dmitry Sanarin, Fredrick Meunier, Philip Kendall
 
-   $Id: trdos.c,v 1.13 2003/07/11 11:03:25 pak21 Exp $
+   $Id: trdos.c,v 1.21 2004/01/08 17:02:57 pak21 Exp $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -34,6 +34,9 @@
 #include <fcntl.h>
 #include <unistd.h>
 #include <string.h>
+#ifdef HAVE_STRINGS_H
+#include <strings.h>		/* Needed for strncasecmp() on QNX6 */
+#endif				/* #ifdef HAVE_STRINGS_H */
 #include <limits.h>
 #include <sys/stat.h>
 
@@ -158,11 +161,15 @@ union
 
 static discs_type discs[4];
 
+int trdos_available = 0;
 int trdos_active=0;
 
 /* The template used for naming the results of the SCL->TRD conversion */
 #define SCL_TMP_FILE_TEMPLATE "fuse.scl.XXXXXX"
 
+/* The template used for the temporary copy of a .trd file */
+static const char *trd_template = "fuse.trd.XXXXXX";
+
 static int Scl2Trd( const char *oldname, int TRD );
 static int insert_scl( trdos_drive_number which, const char *filename );
 static int insert_trd( trdos_drive_number which, const char *filename );
@@ -184,20 +191,23 @@ void
 trdos_reset( void )
 {
   trdos_active = 0;
+  busy = 0;
 
   trdos_event_index( 0 );
 
   /* We can eject disks only if they are currently present */
-  ui_menu_activate_media_disk_eject( TRDOS_DRIVE_A,
-				     discs[ TRDOS_DRIVE_A ].disc_ready );
-  ui_menu_activate_media_disk_eject( TRDOS_DRIVE_B,
-				     discs[ TRDOS_DRIVE_B ].disc_ready );
+  ui_menu_activate( UI_MENU_ITEM_MEDIA_DISK_A_EJECT,
+		    discs[ TRDOS_DRIVE_A ].disc_ready );
+  ui_menu_activate( UI_MENU_ITEM_MEDIA_DISK_B_EJECT,
+		    discs[ TRDOS_DRIVE_B ].disc_ready );
+
+  ui_statusbar_update( UI_STATUSBAR_ITEM_DISK, UI_STATUSBAR_STATE_INACTIVE );
 }
 
 void
 trdos_end( void )
 {
-  ;
+  trdos_available = 0;
 }
 
 static
@@ -209,6 +219,10 @@ void trdos_update_index_impulse( void )
       vg_rs.bit.b1 = 1;
     }
   }
+
+  ui_statusbar_update( UI_STATUSBAR_ITEM_DISK,
+                       busy ? UI_STATUSBAR_STATE_ACTIVE :
+                              UI_STATUSBAR_STATE_INACTIVE );
 }
 
 libspectrum_byte
@@ -332,70 +346,17 @@ trdos_sp_write( libspectrum_word port GCC_UNUSED, libspectrum_byte b )
 static int
 insert_trd( trdos_drive_number which, const char *filename )
 {
-  int fil;
-  libspectrum_byte *temp;
+  int fd, error;
+  char tempfilename[ PATH_MAX ];
+
+  /* Make a temporary copy of the disk */
+  error = utils_make_temp_file( &fd, tempfilename, filename, trd_template );
+  if( error ) return error;
+
+  unlink( tempfilename );
 
   discs[which].ro = 0;
-
-  fil = open( filename, O_RDWR | O_BINARY );
-  if( fil == -1 ) {
-
-    /* If we couldn't open the file read-write, try opening it read-only */
-    if( errno == EACCES ) {
-      discs[which].ro = 1;
-      fil = open( filename, O_RDONLY | O_BINARY );
-    }
-  }
-
-  if( fil == -1 ) {
-
-    ssize_t written;
-
-    /* If we got an error other than 'file doesn't exist', don't try
-       anything else */
-    if( errno != ENOENT ) {
-      ui_error( UI_ERROR_ERROR, "Couldn't open '%s': %s", filename,
-		strerror( errno ) );
-      return 1;
-    }
-
-    ui_error( UI_ERROR_INFO, "No such file '%s'; creating it", filename );
-    discs[which].ro = 0;
-
-    /* FIXME: NFS locking problems */
-    fil = open( filename, O_CREAT | O_EXCL | O_RDWR | O_BINARY );
-    if( fil == -1 ) {
-      ui_error( UI_ERROR_ERROR, "Couldn't open '%s': %s", filename,
-		strerror( errno ) );
-      return 1;
-    }
-
-    temp = malloc( TRDOS_DISC_SIZE );
-    if( !temp ) {
-      ui_error( UI_ERROR_ERROR, "Out of memory at %s:%d", __FILE__, __LINE__ );
-      close( fil );
-      return 1;
-    }
-
-    memset( temp, TRDOS_DISC_SIZE, 0 );
-
-    written = write( fil, temp, TRDOS_DISC_SIZE );
-    if( written != TRDOS_DISC_SIZE ) {
-      if( written == -1 ) {
-	ui_error( UI_ERROR_ERROR, "Error writing to '%s': %s", filename,
-		  strerror( errno ) );
-      } else {
-	ui_error( UI_ERROR_ERROR,
-		  "Only wrote %lu bytes out of %d to '%s'",
-		  (unsigned long)written, TRDOS_DISC_SIZE, filename );
-      }
-      close( fil );
-      return 1;
-    }
-
-  }
-
-  discs[which].fd = fil;
+  discs[which].fd = fd;
   strcpy( discs[which].filename, filename );
   discs[which].disc_ready = 1;
 
@@ -411,7 +372,9 @@ insert_scl( trdos_drive_number which, const char *filename )
 
   temp_path = utils_get_temp_path();
 
-  length = strlen( temp_path ) + strlen( SCL_TMP_FILE_TEMPLATE ) + 1;
+  /* +2 is for the slash between the path and the template and for the
+     null at the end */
+  length = strlen( temp_path ) + strlen( SCL_TMP_FILE_TEMPLATE ) + 2;
 
   trd_template = malloc( length );
   if( !trd_template ) {
@@ -419,7 +382,7 @@ insert_scl( trdos_drive_number which, const char *filename )
     return 1;
   }
 
-  snprintf( trd_template, length, "%s%s", temp_path, SCL_TMP_FILE_TEMPLATE );
+  snprintf( trd_template, length, "%s/%s", temp_path, SCL_TMP_FILE_TEMPLATE );
 
   discs[ which ].disc_ready = 0;
 
@@ -458,19 +421,17 @@ trdos_disk_insert( trdos_drive_number which, const char *filename )
   libspectrum_class_t class;
 
   if( discs[ which ].disc_ready ) {
-    if( trdos_disk_eject( which ) ) return 1;
+    if( trdos_disk_eject( which, 0 ) ) return 1;
   }
 
   if( utils_read_file( filename, &file ) ) return 1;
 
-  if( libspectrum_identify_file( &type, filename, file.buffer, file.length ) ) {
+  if( libspectrum_identify_file_with_class( &type, &class, filename,
+					    file.buffer, file.length ) ) {
     utils_close_file( &file );
     return 1;
   }
 
-  error = libspectrum_identify_class( &class, type );
-  if( error ) return 1;
-
   if( class != LIBSPECTRUM_CLASS_DISK_TRDOS ) {
     ui_error( UI_ERROR_ERROR,
               "trdos_disk_insert: file `%s' is not a TR-DOS disk", filename );
@@ -495,24 +456,75 @@ trdos_disk_insert( trdos_drive_number which, const char *filename )
   if( error ) return error;
 
   /* Set the `eject' item active */
-  ui_menu_activate_media_disk_eject( which, 1 );
+  ui_menu_activate(
+    which == TRDOS_DRIVE_A ? UI_MENU_ITEM_MEDIA_DISK_A_EJECT :
+			     UI_MENU_ITEM_MEDIA_DISK_B_EJECT  ,
+    1
+  );
 
   return 0;
 }
 
 int
-trdos_disk_eject( trdos_drive_number which )
+trdos_disk_eject( trdos_drive_number which, int write )
 {
-  if( close( discs[which].fd ) == -1 ) {
-    ui_error( UI_ERROR_ERROR, "Error closing '%s': %s", discs[which].filename,
-	      strerror( errno ) );
+  int error;
+
+  if( write ) {
+    ui_trdos_disk_write( which );
+  } else {
+    error = close( discs[ which ].fd );
+    if( error ) {
+      ui_error( UI_ERROR_ERROR, "Error closing '%s': %s",
+		discs[which].filename, strerror( errno ) );
+    }
     return 1;
   }
 
   discs[which].disc_ready = 0;
 
   /* Set the `eject' item inactive */
-  ui_menu_activate_media_disk_eject( which, 0 );
+  ui_menu_activate(
+    which == TRDOS_DRIVE_A ? UI_MENU_ITEM_MEDIA_DISK_A_EJECT :
+			     UI_MENU_ITEM_MEDIA_DISK_B_EJECT  ,
+    0
+  );
+
+  return 0;
+}
+
+int
+trdos_disk_write( trdos_drive_number which, const char *filename )
+{
+  FILE *f;
+  utils_file file;
+  int error;
+  ssize_t bytes_written;
+
+  f = fopen( filename, "wb" );
+  if( !f ) {
+    ui_error( UI_ERROR_ERROR, "couldn't open '%s' for writing: %s", filename,
+	      strerror( errno ) );
+  }
+
+  error = utils_read_fd( discs[ which ].fd, discs[ which ].filename, &file );
+  if( error ) { fclose( f ); return error; }
+
+  bytes_written = fwrite( file.buffer, 1, file.length, f );
+  if( bytes_written != file.length ) {
+    ui_error( UI_ERROR_ERROR, "could write only %lu of %lu bytes to '%s'",
+	      (unsigned long)bytes_written, (unsigned long)file.length,
+	      filename );
+    utils_close_file( &file ); fclose( f );
+  }
+
+  error = utils_close_file( &file ); if( error ) { fclose( f ); return error; }
+
+  if( fclose( f ) ) {
+    ui_error( UI_ERROR_ERROR, "error closing '%s': %s", filename,
+	      strerror( errno ) );
+    return 1;
+  }
 
   return 0;
 }
