@@ -1,7 +1,7 @@
 /* debugger.c: the GTK+ debugger
-   Copyright (c) 2002 Philip Kendall
+   Copyright (c) 2002-2003 Philip Kendall
 
-   $Id: debugger.c,v 1.40 2003/08/14 13:30:37 pak21 Exp $
+   $Id: debugger.c,v 1.54 2004/01/08 23:19:32 pak21 Exp $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -37,18 +37,54 @@
 #include <libspectrum.h>
 
 #include "debugger/debugger.h"
+#include "event.h"
 #include "fuse.h"
-#include "gtkui.h"
+#include "gtkinternals.h"
 #include "machine.h"
+#include "memory.h"
 #include "scld.h"
-#include "spectrum.h"
 #include "ui/ui.h"
 #include "z80/z80.h"
 #include "z80/z80_macros.h"
 
+/* The various debugger panes */
+typedef enum debugger_pane {
+
+  DEBUGGER_PANE_BEGIN = 1,	/* Start marker */
+
+  DEBUGGER_PANE_REGISTERS = DEBUGGER_PANE_BEGIN,
+  DEBUGGER_PANE_BREAKPOINTS,
+  DEBUGGER_PANE_DISASSEMBLY,
+  DEBUGGER_PANE_STACK,
+  DEBUGGER_PANE_EVENTS,
+
+  DEBUGGER_PANE_END		/* End marker */
+
+} debugger_pane;
+
 static int create_dialog( void );
+static int hide_hidden_panes( void );
+static GtkCheckMenuItem* get_pane_menu_item( debugger_pane pane );
+static GtkWidget* get_pane( debugger_pane pane );
+static int create_menu_bar( GtkBox *parent, GtkAccelGroup *accel_group );
+static void toggle_display( gpointer callback_data, guint callback_action,
+			    GtkWidget *widget );
+static int create_register_display( GtkBox *parent, gtkui_font font );
+static int create_breakpoints( GtkBox *parent );
+static int create_disassembly( GtkBox *parent, gtkui_font font );
+static int create_stack_display( GtkBox *parent, gtkui_font font );
+static void stack_click( GtkCList *clist, gint row, gint column,
+			 GdkEventButton *event, gpointer user_data );
+static int create_events( GtkBox *parent );
+static void events_click( GtkCList *clist, gint row, gint column,
+			  GdkEventButton *event, gpointer user_data );
+static int create_command_entry( GtkBox *parent, GtkAccelGroup *accel_group );
+static int create_buttons( GtkContainer *parent, GtkAccelGroup *accel_group );
+
 static int activate_debugger( void );
 static int update_disassembly( void );
+static int update_events( void );
+static void add_event( gpointer data, gpointer user_data );
 static int deactivate_debugger( void );
 
 static void move_disassembly( GtkAdjustment *adjustment, gpointer user_data );
@@ -58,14 +94,19 @@ static void gtkui_debugger_done_step( GtkWidget *widget, gpointer user_data );
 static void gtkui_debugger_done_continue( GtkWidget *widget,
 					  gpointer user_data );
 static void gtkui_debugger_break( GtkWidget *widget, gpointer user_data );
+static gboolean delete_dialog( GtkWidget *widget, GdkEvent *event,
+			       gpointer user_data );
 static void gtkui_debugger_done_close( GtkWidget *widget, gpointer user_data );
 
 static GtkWidget *dialog,		/* The debugger dialog box */
   *continue_button, *break_button,	/* Two of its buttons */
-  *registers[18],			/* The register display */
+  *register_display,			/* The register display */
+  *registers[18],			/* Individual registers */
   *breakpoints,				/* The breakpoint display */
-  *disassembly,				/* The disassembly */
-  *stack;				/* The stack display */
+  *disassembly_box,			/* A box to hold the disassembly */
+  *disassembly,				/* The actual disassembly widget */
+  *stack,				/* The stack display */
+  *events;				/* The events display */
 
 static GtkObject *disassembly_scrollbar_adjustment;
 
@@ -78,15 +119,36 @@ static int dialog_created = 0;
 /* Is the debugger window active (as opposed to the debugger itself)? */
 static int debugger_active;
 
+/* The factory used to create the menu bar */
+static GtkItemFactory *menu_factory;
+
+/* The debugger's menu bar */
+static GtkItemFactoryEntry menu_data[] = {
+
+  { "/_View", NULL, NULL, 0, "<Branch>" },
+  { "/View/_Registers", NULL, toggle_display, DEBUGGER_PANE_REGISTERS, "<ToggleItem>" },
+  { "/View/_Breakpoints", NULL, toggle_display, DEBUGGER_PANE_BREAKPOINTS, "<ToggleItem>" },
+  { "/View/_Disassembly", NULL, toggle_display, DEBUGGER_PANE_DISASSEMBLY, "<ToggleItem>" },
+  { "/View/_Stack", NULL, toggle_display, DEBUGGER_PANE_STACK, "<ToggleItem>" },
+  { "/View/_Events", NULL, toggle_display, DEBUGGER_PANE_EVENTS, "<ToggleItem>" },
+
+};
+
+static guint menu_data_count =
+  sizeof( menu_data ) / sizeof( GtkItemFactoryEntry );
+
 int
 ui_debugger_activate( void )
 {
+  int error;
+
   fuse_emulation_pause();
 
   /* Create the dialog box if it doesn't already exist */
   if( !dialog_created ) if( create_dialog() ) return 1;
 
   gtk_widget_show_all( dialog );
+  error = hide_hidden_panes(); if( error ) return error;
 
   gtk_widget_set_sensitive( continue_button, 1 );
   gtk_widget_set_sensitive( break_button, 0 );
@@ -95,6 +157,76 @@ ui_debugger_activate( void )
   return 0;
 }
 
+static int
+hide_hidden_panes( void )
+{
+  debugger_pane i;
+  GtkCheckMenuItem *checkitem; GtkWidget *pane;
+
+  for( i = DEBUGGER_PANE_BEGIN; i < DEBUGGER_PANE_END; i++ ) {
+
+    checkitem = get_pane_menu_item( i ); if( !checkitem ) return 1;
+
+    if( checkitem->active ) continue;
+
+    pane = get_pane( i ); if( !pane ) return 1;
+
+    gtk_widget_hide_all( pane );
+  }
+
+  return 0;
+}
+
+static GtkCheckMenuItem*
+get_pane_menu_item( debugger_pane pane )
+{
+  const gchar *path;
+  GtkWidget *menu_item;
+
+  path = NULL;
+
+  switch( pane ) {
+  case DEBUGGER_PANE_REGISTERS: path = "/View/Registers"; break;
+  case DEBUGGER_PANE_BREAKPOINTS: path = "/View/Breakpoints"; break;
+  case DEBUGGER_PANE_DISASSEMBLY: path = "/View/Disassembly"; break;
+  case DEBUGGER_PANE_STACK: path = "/View/Stack"; break;
+  case DEBUGGER_PANE_EVENTS: path = "/View/Events"; break;
+
+  case DEBUGGER_PANE_END: break;
+  }
+
+  if( !path ) {
+    ui_error( UI_ERROR_ERROR, "unknown debugger pane %u\n", pane );
+    return NULL;
+  }
+
+  menu_item = gtk_item_factory_get_widget( menu_factory, path );
+  if( !menu_item ) {
+    ui_error( UI_ERROR_ERROR, "couldn't get menu item '%s' from factory",
+	      path );
+    return NULL;
+  }
+
+  return GTK_CHECK_MENU_ITEM( menu_item );
+}
+
+static GtkWidget*
+get_pane( debugger_pane pane )
+{
+  switch( pane ) {
+  case DEBUGGER_PANE_REGISTERS: return register_display;
+  case DEBUGGER_PANE_BREAKPOINTS: return breakpoints;
+  case DEBUGGER_PANE_DISASSEMBLY: return disassembly_box;
+  case DEBUGGER_PANE_STACK: return stack;
+  case DEBUGGER_PANE_EVENTS: return events;
+
+  case DEBUGGER_PANE_END: break;
+  }
+
+  ui_error( UI_ERROR_ERROR, "unknown debugger pane %u\n", pane );
+  return NULL;
+}
+  
 int
 ui_debugger_deactivate( int interruptable )
 {
@@ -109,44 +241,31 @@ ui_debugger_deactivate( int interruptable )
 static int
 create_dialog( void )
 {
-  size_t i;
+  int error;
   GtkWidget *hbox, *vbox;
-  GtkWidget *table;
-  GtkWidget *entry, *eval_button;
-  GtkWidget *step_button, *close_button;
   GtkAccelGroup *accel_group;
-#ifdef UI_GTK2
-  PangoFontDescription *font_desc;
-#else				/* #ifdef UI_GTK2 */
-  GtkStyle *style;
-#endif				/* #ifdef UI_GTK2 */
+  debugger_pane i;
 
-  GtkWidget *scrollbar;
+  gtkui_font font;
 
-  gchar
-    *breakpoint_titles[] = { "ID", "Type", "Value", "Ignore", "Life",
-			     "Condition" },
-    *disassembly_titles[] = { "Address", "Instruction" },
-    *stack_titles[] = { "Address", "Value" };
-
-  /* Try and get a monospaced font */
-#ifdef UI_GTK2
-  font_desc = pango_font_description_from_string( "Monospace 12" );
-  if( !font_desc ) {
-#else				/* #ifdef UI_GTK2 */
-  style = gtk_style_new();
-  gdk_font_unref( style->font );
-
-  style->font = gdk_font_load( "-*-courier-medium-r-*-*-12-*-*-*-*-*-*-*" );
-  if( !style->font ) {
-#endif				/* #ifdef UI_GTK2 */
-    ui_error( UI_ERROR_ERROR, "couldn't find a monospaced font" );
-    return 1;
-  }
+  error = gtkui_get_monospaced_font( &font ); if( error ) return error;
 
   dialog = gtk_dialog_new();
   gtk_window_set_title( GTK_WINDOW( dialog ), "Fuse - Debugger" );
 
+  /* Keyboard shortcuts */
+  accel_group = gtk_accel_group_new();
+  gtk_window_add_accel_group( GTK_WINDOW( dialog ), accel_group );
+
+  /* Catch attempts to close the window and just hide it instead */
+  gtk_signal_connect( GTK_OBJECT( dialog ), "delete-event",
+		      GTK_SIGNAL_FUNC( delete_dialog ), NULL );
+
+  /* The menu bar */
+  error = create_menu_bar( GTK_BOX( GTK_DIALOG( dialog )->vbox ),
+			   accel_group );
+  if( error ) return error;
+
   /* A couple of boxes to contain the things we want to display */
   hbox = gtk_hbox_new( FALSE, 0 );
   gtk_box_pack_start( GTK_BOX( GTK_DIALOG( dialog )->vbox ), hbox,
@@ -155,39 +274,131 @@ create_dialog( void )
   vbox = gtk_vbox_new( FALSE, 5 );
   gtk_box_pack_start( GTK_BOX( hbox ), vbox, TRUE, TRUE, 5 );
 
-  /* 'table' contains the register display */
-  table = gtk_table_new( 9, 2, FALSE );
-  gtk_box_pack_start( GTK_BOX( vbox ), table, FALSE, FALSE, 0 );
+  /* The main display areas */
+  error = create_register_display( GTK_BOX( vbox ), font );
+  if( error ) return error;
+
+  error = create_breakpoints( GTK_BOX( vbox ) ); if( error ) return error;
+
+  error = create_disassembly( GTK_BOX( hbox ), font );
+  if( error ) return error;
+
+  error = create_stack_display( GTK_BOX( hbox ), font );
+  if( error ) return error;
+
+  error = create_events( GTK_BOX( hbox ) ); if( error ) return error;
+
+  error = create_command_entry( GTK_BOX( GTK_DIALOG( dialog )->vbox ),
+				accel_group );
+  if( error ) return error;
+
+  /* The action buttons */
+
+  error = create_buttons( GTK_CONTAINER( GTK_DIALOG( dialog )->action_area ),
+			  accel_group );
+  if( error ) return error;
+
+  /* Initially, have all the panes visible */
+  for( i = DEBUGGER_PANE_BEGIN; i < DEBUGGER_PANE_END; i++ ) {
+    
+    GtkCheckMenuItem *check_item;
+
+    check_item = get_pane_menu_item( i ); if( !check_item ) break;
+    gtk_check_menu_item_set_active( check_item, TRUE );
+  }
+
+  gtkui_free_font( font );
+
+  dialog_created = 1;
+
+  return 0;
+}
+
+static int
+create_menu_bar( GtkBox *parent, GtkAccelGroup *accel_group )
+{
+  GtkWidget *menu_bar;
+
+  menu_factory = gtk_item_factory_new( GTK_TYPE_MENU_BAR, "<main>",
+				       accel_group );
+  gtk_item_factory_create_items( menu_factory, menu_data_count, menu_data,
+				 NULL);
+  menu_bar = gtk_item_factory_get_widget( menu_factory, "<main>" );
+
+  gtk_box_pack_start_defaults( parent, menu_bar );
+  
+  return 0;
+}
+
+static void
+toggle_display( gpointer callback_data, guint callback_action,
+		GtkWidget *widget )
+{
+  GtkWidget *pane;
+
+  pane = get_pane( callback_action ); if( !pane ) return;
+
+  if( GTK_CHECK_MENU_ITEM( widget )->active ) {
+    gtk_widget_show_all( pane );
+  } else {
+    gtk_widget_hide_all( pane );
+  }
+}
+
+static int
+create_register_display( GtkBox *parent, gtkui_font font )
+{
+  size_t i;
+
+  register_display = gtk_table_new( 9, 2, FALSE );
+  gtk_box_pack_start( parent, register_display, FALSE, FALSE, 0 );
 
   for( i = 0; i < 18; i++ ) {
     registers[i] = gtk_label_new( "" );
-#ifdef UI_GTK2
-    gtk_widget_modify_font( registers[i], font_desc );
-#else				/* #ifdef UI_GTK2 */
-    gtk_widget_set_style( registers[i], style );
-#endif				/* #ifdef UI_GTK2 */
-    gtk_table_attach( GTK_TABLE( table ), registers[i], i%2, i%2+1, i/2, i/2+1,
-		      0, 0, 2, 2 );
+    gtkui_set_font( registers[i], font );
+    gtk_table_attach( GTK_TABLE( register_display ), registers[i],
+		      i%2, i%2+1, i/2, i/2+1, 0, 0, 2, 2 );
   }
 
-  /* The breakpoint CList */
+  return 0;
+}
+
+static int
+create_breakpoints( GtkBox *parent )
+{
+  size_t i;
+
+  gchar *breakpoint_titles[] = { "ID", "Type", "Value", "Ignore", "Life",
+				 "Condition" };
+
   breakpoints = gtk_clist_new_with_titles( 6, breakpoint_titles );
   gtk_clist_column_titles_passive( GTK_CLIST( breakpoints ) );
   for( i = 0; i < 6; i++ )
     gtk_clist_set_column_auto_resize( GTK_CLIST( breakpoints ), i, TRUE );
-  gtk_box_pack_start_defaults( GTK_BOX( vbox ), breakpoints );
+  gtk_box_pack_start_defaults( parent, breakpoints );
 
-  /* Create the disassembly CList itself */
+  return 0;
+}
+
+static int
+create_disassembly( GtkBox *parent, gtkui_font font )
+{
+  size_t i;
+
+  GtkWidget *scrollbar;
+  gchar *disassembly_titles[] = { "Address", "Instruction" };
+
+  /* A box to hold the disassembly listing and the scrollbar */
+  disassembly_box = gtk_hbox_new( FALSE, 0 );
+  gtk_box_pack_start_defaults( parent, disassembly_box );
+
+  /* The disassembly CList itself */
   disassembly = gtk_clist_new_with_titles( 2, disassembly_titles );
-#ifdef UI_GTK2
-  gtk_widget_modify_font( disassembly, font_desc );
-#else				/* #ifdef UI_GTK2 */
-  gtk_widget_set_style( disassembly, style );
-#endif				/* #ifdef UI_GTK2 */
+  gtkui_set_font( disassembly, font );
   gtk_clist_column_titles_passive( GTK_CLIST( disassembly ) );
   for( i = 0; i < 2; i++ )
     gtk_clist_set_column_auto_resize( GTK_CLIST( disassembly ), i, TRUE );
-  gtk_box_pack_start_defaults( GTK_BOX( hbox ), disassembly );
+  gtk_box_pack_start_defaults( GTK_BOX( disassembly_box ), disassembly );
 
   /* The disassembly scrollbar */
   disassembly_scrollbar_adjustment =
@@ -197,25 +408,106 @@ create_dialog( void )
 		      NULL );
   scrollbar =
     gtk_vscrollbar_new( GTK_ADJUSTMENT( disassembly_scrollbar_adjustment ) );
-  gtk_box_pack_start( GTK_BOX( hbox ), scrollbar, FALSE, FALSE, 0 );
+  gtk_box_pack_start( GTK_BOX( disassembly_box ), scrollbar, FALSE, FALSE, 0 );
+
+  return 0;
+}
+
+static int
+create_stack_display( GtkBox *parent, gtkui_font font )
+{
+  size_t i;
+  gchar *stack_titles[] = { "Address", "Value" };
 
-  /* And the stack CList */
   stack = gtk_clist_new_with_titles( 2, stack_titles );
-#ifdef UI_GTK2
-  gtk_widget_modify_font( stack, font_desc );
-#else				/* #ifdef UI_GTK2 */
-  gtk_widget_set_style( stack, style );
-#endif				/* #ifdef UI_GTK2 */
+  gtkui_set_font( stack, font );
   gtk_clist_column_titles_passive( GTK_CLIST( stack ) );
   for( i = 0; i < 2; i++ )
     gtk_clist_set_column_auto_resize( GTK_CLIST( stack ), i, TRUE );
-  gtk_box_pack_start( GTK_BOX( hbox ), stack, TRUE, TRUE, 5 );
+  gtk_box_pack_start( parent, stack, TRUE, TRUE, 5 );
 
-  /* Another hbox to hold the command entry widget and the 'evaluate'
-     button */
+  gtk_signal_connect( GTK_OBJECT( stack ), "select-row",
+		      GTK_SIGNAL_FUNC( stack_click ), NULL );
+
+  return 0;
+}
+
+static void
+stack_click( GtkCList *clist, gint row, gint column,
+	     GdkEventButton *event, gpointer user_data )
+{
+  libspectrum_word address, destination;
+  int error;
+
+  /* Ignore events which aren't a double-click */
+  if( event->type != GDK_2BUTTON_PRESS ) return;
+
+  address = SP + ( 19 - row ) * 2;
+
+  destination =
+    readbyte_internal( address ) + readbyte_internal( address + 1 ) * 0x100;
+
+  error = debugger_breakpoint_add( DEBUGGER_BREAKPOINT_TYPE_EXECUTE,
+				   -1, destination, 0,
+				   DEBUGGER_BREAKPOINT_LIFE_ONESHOT, NULL );
+  if( error ) return;
+
+  debugger_run();
+}
+
+static int
+create_events( GtkBox *parent )
+{
+  size_t i;
+  gchar *titles[] = { "Time", "Type" };
+
+  events = gtk_clist_new_with_titles( 2, titles );
+  gtk_clist_column_titles_passive( GTK_CLIST( events ) );
+  for( i = 0; i < 2; i++ )
+    gtk_clist_set_column_auto_resize( GTK_CLIST( events ), i, TRUE );
+  gtk_box_pack_start( parent, events, TRUE, TRUE, 5 );
+
+  gtk_signal_connect( GTK_OBJECT( events ), "select-row",
+		      GTK_SIGNAL_FUNC( events_click ), NULL );
+
+  return 0;
+}
+
+static void
+events_click( GtkCList *clist, gint row, gint column,
+	      GdkEventButton *event, gpointer user_data )
+{
+  int got_text, error;
+  gchar *buffer;
+  libspectrum_dword tstates;
+
+  /* Ignore events which aren't a double-click */
+  if( event->type != GDK_2BUTTON_PRESS ) return;
+
+  got_text = gtk_clist_get_text( clist, row, 0, &buffer );
+  if( !got_text ) {
+    ui_error( UI_ERROR_ERROR, "couldn't get text for row %d", row );
+    return;
+  }
+
+  tstates = atoi( buffer );
+
+  error = debugger_breakpoint_add( DEBUGGER_BREAKPOINT_TYPE_TIME,
+				   -1, tstates, 0,
+				   DEBUGGER_BREAKPOINT_LIFE_ONESHOT, NULL );
+  if( error ) return;
+
+  debugger_run();
+}
+
+static int
+create_command_entry( GtkBox *parent, GtkAccelGroup *accel_group )
+{
+  GtkWidget *hbox, *entry, *eval_button;
+
+  /* An hbox to hold the command entry widget and the 'evaluate' button */
   hbox = gtk_hbox_new( FALSE, 5 );
-  gtk_box_pack_start( GTK_BOX( GTK_DIALOG( dialog )->vbox ), hbox,
-		      FALSE, FALSE, 0 );
+  gtk_box_pack_start( parent, hbox, FALSE, FALSE, 0 );
 
   /* The command entry widget */
   entry = gtk_entry_new();
@@ -230,56 +522,45 @@ create_dialog( void )
 			     GTK_OBJECT( entry ) );
   gtk_box_pack_start( GTK_BOX( hbox ), eval_button, FALSE, FALSE, 0 );
 
-  /* The action buttons for the dialog box */
+  /* Return is equivalent to clicking on 'evaluate' */
+  gtk_widget_add_accelerator( eval_button, "clicked", accel_group,
+			      GDK_Return, 0, 0 );
+
+  return 0;
+}
+
+static int
+create_buttons( GtkContainer *parent, GtkAccelGroup *accel_group )
+{
+  GtkWidget *step_button, *close_button;
+
+  /* Create the action buttons for the dialog box */
 
   step_button = gtk_button_new_with_label( "Single Step" );
   gtk_signal_connect( GTK_OBJECT( step_button ), "clicked",
 		      GTK_SIGNAL_FUNC( gtkui_debugger_done_step ), NULL );
-  gtk_container_add( GTK_CONTAINER( GTK_DIALOG( dialog )->action_area ),
-		     step_button );
+  gtk_container_add( parent, step_button );
 
   continue_button = gtk_button_new_with_label( "Continue" );
   gtk_signal_connect( GTK_OBJECT( continue_button ), "clicked",
 		      GTK_SIGNAL_FUNC( gtkui_debugger_done_continue ), NULL );
-  gtk_container_add( GTK_CONTAINER( GTK_DIALOG( dialog )->action_area ),
-		     continue_button );
+  gtk_container_add( parent, continue_button );
 
   break_button = gtk_button_new_with_label( "Break" );
   gtk_signal_connect( GTK_OBJECT( break_button ), "clicked",
 		      GTK_SIGNAL_FUNC( gtkui_debugger_break ), NULL );
-  gtk_container_add( GTK_CONTAINER( GTK_DIALOG( dialog )->action_area ),
-		     break_button );
+  gtk_container_add( parent, break_button );
 
   close_button = gtk_button_new_with_label( "Close" );
   gtk_signal_connect_object( GTK_OBJECT( close_button ), "clicked",
 			     GTK_SIGNAL_FUNC( gtkui_debugger_done_close ),
 			     GTK_OBJECT( dialog ) );
-  gtk_container_add( GTK_CONTAINER( GTK_DIALOG( dialog )->action_area ),
-		     close_button );
-
-  /* Deleting the window is the same as pressing 'Close' */
-  gtk_signal_connect( GTK_OBJECT( dialog ), "delete_event",
-		      GTK_SIGNAL_FUNC( gtkui_debugger_done_close ),
-		      (gpointer) NULL );
-
-  /* Keyboard shortcuts */
-  accel_group = gtk_accel_group_new();
-  gtk_window_add_accel_group( GTK_WINDOW( dialog ), accel_group );
-
-  /* Return is equivalent to clicking on 'evaluate' */
-  gtk_widget_add_accelerator( eval_button, "clicked", accel_group,
-			      GDK_Return, 0, 0 );
+  gtk_container_add( parent, close_button );
 
   /* Esc is equivalent to clicking on 'close' */
   gtk_widget_add_accelerator( close_button, "clicked", accel_group,
                               GDK_Escape, 0, 0 );
 
-#ifdef UI_GTK2
-  pango_font_description_free( font_desc );
-#endif				/* #ifdef UI_GTK2 */
-
-  dialog_created = 1;
-
   return 0;
 }
 
@@ -323,6 +604,8 @@ ui_debugger_update( void )
 				    &HL, &HL_, &IX, &IY,
 				  };
 
+  if( !dialog_created ) return 0;
+
   if( debugger_output_base == 10 ) {
     format_16_bit = "%5d"; format_8_bit = "%3d";
   } else {
@@ -403,14 +686,25 @@ ui_debugger_update( void )
     snprintf( breakpoint_text[0], 40, "%lu", (unsigned long)bp->id );
     snprintf( breakpoint_text[1], 40, "%s",
 	      debugger_breakpoint_type_text[ bp->type ] );
-    snprintf( breakpoint_text[2], 40, format_16_bit, bp->value );
+
+    if( bp->type == DEBUGGER_BREAKPOINT_TYPE_TIME ) {
+      snprintf( breakpoint_text[2], 40, "%5d", bp->value );
+    } else {
+      if( bp->page == -1 ) {
+	snprintf( breakpoint_text[2], 40, format_16_bit, bp->value );
+      } else {
+	sprintf( format_string, "%%d:%s", format_16_bit );
+	snprintf( breakpoint_text[2], 40, format_string, bp->page, bp->value );
+      }
+    }
+
     snprintf( breakpoint_text[3], 40, "%lu", (unsigned long)bp->ignore );
     snprintf( breakpoint_text[4], 40, "%s",
 	      debugger_breakpoint_life_text[ bp->life ] );
     if( bp->condition ) {
       debugger_expression_deparse( breakpoint_text[5], 80, bp->condition );
     } else {
-      breakpoint_text[5] = "";
+      strcpy( breakpoint_text[5], "" );
     }
 
     gtk_clist_append( GTK_CLIST( breakpoints ), breakpoint_text );
@@ -437,6 +731,9 @@ ui_debugger_update( void )
 
   gtk_clist_thaw( GTK_CLIST( stack ) );
 
+  /* And the events display */
+  error = update_events(); if( error ) return error;
+
   return 0;
 }
 
@@ -466,6 +763,36 @@ update_disassembly( void )
   return 0;
 }
 
+static int
+update_events( void )
+{
+  gtk_clist_freeze( GTK_CLIST( events ) );
+  gtk_clist_clear( GTK_CLIST( events ) );
+
+  event_foreach( add_event, NULL );
+
+  gtk_clist_thaw( GTK_CLIST( events ) );
+
+  return 0;
+}
+
+static void
+add_event( gpointer data, gpointer user_data )
+{
+  event_t *ptr = data;
+
+  char buffer[80];
+  char *event_text[2] = { &buffer[0], &buffer[40] };
+
+  /* Skip events which have been removed */
+  if( ptr->type == EVENT_TYPE_NULL ) return;
+
+  snprintf( event_text[0], 40, "%d", ptr->tstates );
+  strncpy( event_text[1], event_name( ptr->type ), 40 );
+
+  gtk_clist_append( GTK_CLIST( events ), event_text );
+}
+
 static int
 deactivate_debugger( void )
 {
@@ -573,6 +900,14 @@ gtkui_debugger_break( GtkWidget *widget GCC_UNUSED,
   gtk_widget_set_sensitive( break_button, 0 );
 }
 
+static gboolean
+delete_dialog( GtkWidget *widget, GdkEvent *event GCC_UNUSED,
+	       gpointer user_data )
+{
+  gtkui_debugger_done_close( widget, user_data );
+  return TRUE;
+}
+
 static void
 gtkui_debugger_done_close( GtkWidget *widget, gpointer user_data GCC_UNUSED )
 {
