@@ -1,7 +1,7 @@
 /* error.c: GTK+ routines for producing the error dialog box
    Copyright (c) 2002 Philip Kendall
 
-   $Id: error.c,v 1.11 2002/12/13 22:05:01 pak21 Exp $
+   $Id: error.c,v 1.12 2003/10/27 21:00:00 pak21 Exp $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -109,7 +109,7 @@ ui_verror( ui_error_level severity, const char *format, va_list ap )
   gtk_signal_connect_object( GTK_OBJECT( ok_button ), "clicked",
 			     GTK_SIGNAL_FUNC( gtk_widget_destroy ),
 			     GTK_OBJECT( dialog ) );
-  gtk_signal_connect( GTK_OBJECT( dialog ), "delete_event",
+  gtk_signal_connect( GTK_OBJECT( dialog ), "delete-event",
 		      GTK_SIGNAL_FUNC( gtk_widget_destroy ), (gpointer) NULL );
 
   accel_group = gtk_accel_group_new();
