@@ -1,7 +1,7 @@
 /* gtkui.c: GTK+ routines for dealing with the user interface
-   Copyright (c) 2000-2002 Philip Kendall, Russell Marks
+   Copyright (c) 2000-2004 Philip Kendall, Russell Marks
 
-   $Id: gtkui.c,v 1.100 2003/06/18 12:51:10 pak21 Exp $
+   $Id: gtkui.c,v 1.123.2.2 2004/01/31 16:40:58 pak21 Exp $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -29,7 +29,6 @@
 #ifdef UI_GTK		/* Use this file iff we're using GTK+ */
 
 #include <stdio.h>
-#include <string.h>
 
 #include <gdk/gdkkeysyms.h>
 #include <gdk/gdkx.h>
@@ -44,11 +43,10 @@
 #include "display.h"
 #include "event.h"
 #include "fuse.h"
-#include "gtkdisplay.h"
-#include "gtkkeyboard.h"
-#include "gtkui.h"
+#include "gtkinternals.h"
 #include "machine.h"
 #include "options.h"
+#include "pokefinder/pokefinder.h"
 #include "psg.h"
 #include "rzx.h"
 #include "screenshot.h"
@@ -79,6 +77,9 @@ GtkItemFactory *menu_factory;
 /* And that used to create the popup menus */
 GtkItemFactory *popup_factory;
 
+/* True if we were paused via the Machine/Pause menu item */
+static int paused = 0;
+
 /* Structure used by the radio button selection widgets (the graphics
    filter selectors and Machine/Select) */
 typedef struct gtkui_select_info {
@@ -97,8 +98,8 @@ static gboolean gtkui_make_menu(GtkAccelGroup **accel_group,
 				GtkItemFactoryEntry *menu_data,
 				guint menu_data_size);
 
-static gint gtkui_delete(GtkWidget *widget, GdkEvent *event,
-			      gpointer data);
+static gboolean gtkui_delete( GtkWidget *widget, GdkEvent *event,
+			      gpointer data );
 static void gtkui_open(GtkWidget *widget, gpointer data);
 static void gtkui_save(GtkWidget *widget, gpointer data);
 
@@ -126,6 +127,7 @@ static void select_filter_done( GtkWidget *widget, gpointer user_data );
 static void save_options( GtkWidget *widget, gpointer data );
 #endif				/* #ifdef HAVE_LIB_XML2 */
 
+static void gtkui_pause( GtkWidget *widget, gpointer data );
 static void gtkui_reset(GtkWidget *widget, gpointer data);
 
 static void gtkui_select(GtkWidget *widget, gpointer data);
@@ -140,21 +142,15 @@ static void gtkui_tape_rewind( GtkWidget *widget, gpointer data );
 static void gtkui_tape_clear( GtkWidget *widget, gpointer data );
 static void gtkui_tape_write( GtkWidget *widget, gpointer data );
 
-static void gtkui_disk_open_a( GtkWidget *widget, gpointer data );
-static void gtkui_disk_open_b( GtkWidget *widget, gpointer data );
-static void gtkui_disk_eject_a( GtkWidget *widget, gpointer data );
-static void gtkui_disk_eject_b( GtkWidget *widget, gpointer data );
-
+static void disk_open( gpointer data, guint action, GtkWidget *widget );
 static void gtkui_disk_open( specplus3_drive_number drive );
+static void disk_eject( gpointer data, guint action, GtkWidget *widget );
 
 static void cartridge_insert( GtkWidget *widget, gpointer data );
 static void cartridge_eject( GtkWidget *widget, gpointer data );
 
 static void gtkui_help_keyboard( GtkWidget *widget, gpointer data );
 
-static void gtkui_fileselector_done( GtkButton *button, gpointer user_data );
-static void gtkui_fileselector_cancel( GtkButton *button, gpointer user_data );
-
 /* Set a menu item (in)active in both the menu bar and the popup menus */
 static int set_menu_item_active( const char *path, int active );
 
@@ -199,9 +195,12 @@ static GtkItemFactoryEntry gtkui_menu_data[] = {
 #endif				/* #ifdef HAVE_LIB_XML2 */
 
   { "/Machine",		        NULL , NULL,                0, "<Branch>"    },
+  { "/Machine/_Pause...",       "Pause", gtkui_pause,	    0, NULL          },
   { "/Machine/_Reset...",       "F5" , gtkui_reset,         0, NULL          },
   { "/Machine/_Select...",      "F9" , gtkui_select,        0, NULL          },
   { "/Machine/_Debugger...",	NULL , gtkui_break,	    0, NULL          },
+  { "/Machine/P_oke Finder...",	NULL , gtkui_pokefinder,    0, NULL	     },
+  { "/Machine/_Memory Browser...", NULL, gtkui_memory_browser, 0, NULL       },
   { "/Machine/_NMI",		NULL , gtkui_nmi,	    0, NULL          },
 
   { "/Media",			NULL , NULL,                0, "<Branch>"    },
@@ -216,15 +215,13 @@ static GtkItemFactoryEntry gtkui_menu_data[] = {
 
   { "/Media/_Disk",		NULL , NULL,		    0, "<Branch>"    },
   { "/Media/Disk/Drive A:",	NULL , NULL,		    0, "<Branch>"    },
-  { "/Media/Disk/Drive A:/_Insert...",
-				NULL , gtkui_disk_open_a,   0, NULL          },
-  { "/Media/Disk/Drive A:/_Eject",
-				NULL , gtkui_disk_eject_a,  0, NULL          },
+  { "/Media/Disk/Drive A:/_Insert...", NULL, disk_open,     1, NULL          },
+  { "/Media/Disk/Drive A:/_Eject", NULL, disk_eject,        1, NULL          },
+  { "/Media/Disk/Drive A:/Eject and _write...", NULL, disk_eject, 3, NULL    },
   { "/Media/Disk/Drive B:",	NULL , NULL,		    0, "<Branch>"    },
-  { "/Media/Disk/Drive B:/_Insert...",
-				NULL , gtkui_disk_open_b,   0, NULL          },
-  { "/Media/Disk/Drive B:/_Eject",
-				NULL , gtkui_disk_eject_b,  0, NULL          },
+  { "/Media/Disk/Drive B:/_Insert...", NULL, disk_open,     2, NULL          },
+  { "/Media/Disk/Drive B:/_Eject", NULL, disk_eject,        2, NULL          },
+  { "/Media/Disk/Drive B:/Eject and _write...", NULL, disk_eject, 4, NULL    },
 
   { "/Media/_Cartridge",	NULL , NULL,		    0, "<Branch>"    },
   { "/Media/Cartridge/_Insert...",
@@ -240,7 +237,7 @@ static guint gtkui_menu_data_size =
 int
 ui_init( int *argc, char ***argv )
 {
-  GtkWidget *box,*menu_bar;
+  GtkWidget *box, *menu_bar;
   GtkAccelGroup *accel_group;
   GdkGeometry geometry;
   GdkWindowHints hints;
@@ -260,7 +257,7 @@ ui_init( int *argc, char ***argv )
   gtk_window_set_default_size( GTK_WINDOW(gtkui_window),
 			       DISPLAY_ASPECT_WIDTH, DISPLAY_SCREEN_HEIGHT );
 
-  gtk_signal_connect(GTK_OBJECT(gtkui_window), "delete_event",
+  gtk_signal_connect(GTK_OBJECT(gtkui_window), "delete-event",
 		     GTK_SIGNAL_FUNC(gtkui_delete), NULL);
   gtk_signal_connect(GTK_OBJECT(gtkui_window), "key-press-event",
 		     GTK_SIGNAL_FUNC(gtkkeyboard_keypress), NULL);
@@ -294,13 +291,16 @@ ui_init( int *argc, char ***argv )
 
   gtk_box_pack_start( GTK_BOX(box), gtkui_drawing_area, FALSE, FALSE, 0 );
 
+  /* Create the statusbar */
+  gtkstatusbar_create( GTK_BOX( box ) );
+
   hints = GDK_HINT_MIN_SIZE | GDK_HINT_MAX_SIZE |
           GDK_HINT_BASE_SIZE | GDK_HINT_RESIZE_INC;
 
   geometry.min_width = DISPLAY_ASPECT_WIDTH;
   geometry.min_height = DISPLAY_SCREEN_HEIGHT;
-  geometry.max_width = 2 * DISPLAY_ASPECT_WIDTH;
-  geometry.max_height = 2 * DISPLAY_SCREEN_HEIGHT;
+  geometry.max_width = 3 * DISPLAY_ASPECT_WIDTH;
+  geometry.max_height = 3 * DISPLAY_SCREEN_HEIGHT;
   geometry.base_width = 0;
   geometry.base_height = 0;
   geometry.width_inc = DISPLAY_ASPECT_WIDTH;
@@ -318,6 +318,7 @@ ui_init( int *argc, char ***argv )
   if( gtkdisplay_init() ) return 1;
 
   gtk_widget_show_all( gtkui_window );
+  gtkstatusbar_set_visibility( settings_current.statusbar );
 
   return 0;
 }
@@ -341,10 +342,10 @@ static gboolean gtkui_make_menu(GtkAccelGroup **accel_group,
   gtkui_menu_popup = gtk_item_factory_get_widget( popup_factory, "<main>" );
 
   /* Start the recording menu off in the 'not playing' state */
-  ui_menu_activate_recording( 0 );
+  ui_menu_activate( UI_MENU_ITEM_RECORDING, 0 );
 
   /* Start the AY logging menu off in the 'not playing' state */
-  ui_menu_activate_ay_logging( 0 );
+  ui_menu_activate( UI_MENU_ITEM_AY_LOGGING, 0 );
 
   return FALSE;
 }
@@ -392,12 +393,11 @@ int ui_end(void)
 
 /* The callbacks used by various routines */
 
-/* Called by the main window on a "delete_event" */
-static gint
-gtkui_delete( GtkWidget *widget GCC_UNUSED, GdkEvent *event GCC_UNUSED,
-	      gpointer data GCC_UNUSED )
+/* Called by the main window on a "delete-event" */
+static gboolean
+gtkui_delete( GtkWidget *widget, GdkEvent *event GCC_UNUSED, gpointer data )
 {
-  fuse_exiting=1;
+  gtkui_quit( widget, data );
   return TRUE;
 }
 
@@ -479,7 +479,7 @@ gtkui_rzx_start_snap( GtkWidget *widget GCC_UNUSED, gpointer data GCC_UNUSED )
     free( snap ); free( recording ); fuse_emulation_unpause(); return;
   }
 
-  rzx_start_recording( recording, 0 );
+  rzx_start_recording( recording, settings_current.embed_snapshot );
 
   free( recording );
 
@@ -515,14 +515,14 @@ gtkui_rzx_play( GtkWidget *widget GCC_UNUSED, gpointer data GCC_UNUSED )
 
   display_refresh_all();
 
-  ui_menu_activate_recording( 1 );
+  ui_menu_activate( UI_MENU_ITEM_RECORDING, 1 );
 
   fuse_emulation_unpause();
 }
 
 /* Called when File/AY Logging/Record selected */
 static void
-gtkui_psg_start( GtkWidget *widget, gpointer data )
+gtkui_psg_start( GtkWidget *widget GCC_UNUSED, gpointer data GCC_UNUSED )
 {
   char *psgfile;
 
@@ -539,19 +539,19 @@ gtkui_psg_start( GtkWidget *widget, gpointer data )
 
   display_refresh_all();
 
-  ui_menu_activate_ay_logging( 1 );
+  ui_menu_activate( UI_MENU_ITEM_AY_LOGGING, 1 );
 
   fuse_emulation_unpause();
 }
 
 /* Called when File/AY Logging/Stop selected */
 static void
-gtkui_psg_stop( GtkWidget *widget, gpointer data )
+gtkui_psg_stop( GtkWidget *widget GCC_UNUSED, gpointer data GCC_UNUSED )
 {
   if ( !psg_recording ) return;
   psg_stop_recording();
 
-  ui_menu_activate_ay_logging( 0 );
+  ui_menu_activate( UI_MENU_ITEM_AY_LOGGING, 0 );
 
   return;
 }
@@ -641,7 +641,18 @@ gtkui_save_screen( GtkWidget *widget GCC_UNUSED, gpointer data GCC_UNUSED )
 static void
 gtkui_quit( GtkWidget *widget GCC_UNUSED, gpointer data GCC_UNUSED )
 {
-  if( gtkui_confirm( "Exit Fuse?" ) ) fuse_exiting = 1;
+  int error;
+
+  if( gtkui_confirm( "Exit Fuse?" ) ) {
+
+    error = tape_close(); if( error ) return;
+
+    fuse_exiting = 1;
+
+    /* Stop the paused state to allow us to exit (occurs from main
+       emulation loop) */
+    if( paused ) gtkui_pause( NULL, NULL );
+  }
 }
 
 /* Called by the menu when Options/Filter selected */
@@ -729,7 +740,7 @@ ui_get_scaler( ui_scaler_available available )
   gtk_signal_connect_object( GTK_OBJECT( cancel_button ), "clicked",
 			     GTK_SIGNAL_FUNC( gtkui_destroy_widget_and_quit ),
 			     GTK_OBJECT( dialog.dialog ) );
-  gtk_signal_connect( GTK_OBJECT( dialog.dialog ), "delete_event",
+  gtk_signal_connect( GTK_OBJECT( dialog.dialog ), "delete-event",
 		      GTK_SIGNAL_FUNC( gtkui_destroy_widget_and_quit ),
 		      (gpointer) NULL );
 
@@ -787,6 +798,34 @@ save_options( GtkWidget *widget GCC_UNUSED, gpointer data GCC_UNUSED )
 }
 #endif				/* #ifdef HAVE_LIB_XML2 */
 
+/* Machine/Pause */
+static void
+gtkui_pause( GtkWidget *widget GCC_UNUSED, gpointer data GCC_UNUSED )
+{
+  int error;
+
+  if( paused ) {
+    paused = 0;
+    ui_statusbar_update( UI_STATUSBAR_ITEM_PAUSED,
+			 UI_STATUSBAR_STATE_INACTIVE );
+    timer_estimate_reset();
+    timer_count = 0.0;
+    gtk_main_quit();
+  } else {
+
+    /* Stop recording any competition mode RZX file */
+    if( rzx_recording && rzx_competition_mode ) {
+      ui_error( UI_ERROR_INFO, "Stopping competition mode RZX recording" );
+      error = rzx_stop_recording(); if( error ) return;
+    }
+      
+    paused = 1;
+    ui_statusbar_update( UI_STATUSBAR_ITEM_PAUSED, UI_STATUSBAR_STATE_ACTIVE );
+    gtk_main();
+  }
+
+}
+
 /* Called by the menu when Machine/Reset selected */
 static void
 gtkui_reset( GtkWidget *widget GCC_UNUSED, gpointer data GCC_UNUSED )
@@ -860,7 +899,7 @@ gtkui_select( GtkWidget *widget GCC_UNUSED, gpointer data GCC_UNUSED )
   gtk_signal_connect_object( GTK_OBJECT( cancel_button ), "clicked",
 			     GTK_SIGNAL_FUNC( gtkui_destroy_widget_and_quit ),
 			     GTK_OBJECT( dialog.dialog ) );
-  gtk_signal_connect( GTK_OBJECT( dialog.dialog ), "delete_event",
+  gtk_signal_connect( GTK_OBJECT( dialog.dialog ), "delete-event",
 		      GTK_SIGNAL_FUNC( gtkui_destroy_widget_and_quit ),
 		      (gpointer) NULL );
 
@@ -908,6 +947,7 @@ static void
 gtkui_break( GtkWidget *widget GCC_UNUSED, gpointer data GCC_UNUSED )
 {
   debugger_mode = DEBUGGER_MODE_HALTED;
+  if( paused ) ui_debugger_activate();
 }
 
 /* Machine/NMI */
@@ -960,32 +1000,39 @@ gtkui_tape_clear( GtkWidget *widget GCC_UNUSED, gpointer data GCC_UNUSED )
 /* Called by the menu when Tape/Write selected */
 static void
 gtkui_tape_write( GtkWidget *widget GCC_UNUSED, gpointer data GCC_UNUSED )
+{
+  ui_tape_write();
+}
+
+int
+ui_tape_write( void )
 {
   char *filename;
 
   fuse_emulation_pause();
 
   filename = gtkui_fileselector_get_filename( "Fuse - Write Tape" );
-  if( !filename ) { fuse_emulation_unpause(); return; }
+  if( !filename ) { fuse_emulation_unpause(); return 1; }
 
   tape_write( filename );
 
   free( filename );
 
   fuse_emulation_unpause();
+
+  return 0;
 }
 
-/* Called by the mnu when Disk/Drive ?:/Open selected */
+/* Called by the menu when Disk/Drive ?:/Open selected */
 static void
-gtkui_disk_open_a( GtkWidget *widget GCC_UNUSED, gpointer data GCC_UNUSED )
+disk_open( gpointer data GCC_UNUSED, guint action,
+	   GtkWidget *widget GCC_UNUSED )
 {
-  gtkui_disk_open( SPECPLUS3_DRIVE_A );
-}
+  guint which;
 
-static void
-gtkui_disk_open_b( GtkWidget *widget GCC_UNUSED, gpointer data GCC_UNUSED )
-{
-  gtkui_disk_open( SPECPLUS3_DRIVE_B );
+  which = action - 1;
+
+  gtkui_disk_open( which );
 }
 
 static void gtkui_disk_open( specplus3_drive_number drive )
@@ -1012,29 +1059,76 @@ static void gtkui_disk_open( specplus3_drive_number drive )
 }
 
 static void
-gtkui_disk_eject_a( GtkWidget *widget GCC_UNUSED, gpointer data GCC_UNUSED )
+disk_eject( gpointer data GCC_UNUSED, guint action,
+	    GtkWidget *widget GCC_UNUSED )
 {
+  guint parameter, which, write;
+
+  parameter = action - 1;
+  which = parameter & 0x01;
+  write = parameter & 0x02;
+
 #ifdef HAVE_765_H
   if( machine_current->machine == LIBSPECTRUM_MACHINE_PLUS3 ) {
-    specplus3_disk_eject( SPECPLUS3_DRIVE_A );
+    specplus3_disk_eject( which, write );
     return;
   }
 #endif				/* #ifdef HAVE_765_H */
 
-  trdos_disk_eject( TRDOS_DRIVE_A );
+  trdos_disk_eject( which, write );
 }
 
-static void
-gtkui_disk_eject_b( GtkWidget *widget GCC_UNUSED, gpointer data GCC_UNUSED )
-{
 #ifdef HAVE_765_H
-  if( machine_current->machine == LIBSPECTRUM_MACHINE_PLUS3 ) {
-    specplus3_disk_eject( SPECPLUS3_DRIVE_B );
-    return;
-  }
-#endif				/* #ifdef HAVE_765_H */
+int
+ui_plus3_disk_write( specplus3_drive_number which )
+{
+  char drive, *filename, title[80];
 
-  trdos_disk_eject( TRDOS_DRIVE_B );
+  drive = which == SPECPLUS3_DRIVE_A ? 'A' : 'B';
+
+  if( !specplus3_disk_present( which ) ) {
+    ui_error( UI_ERROR_INFO, "No disk present in drive %c:", drive );
+    return 0;
+  }
+
+  fuse_emulation_pause();
+
+  snprintf( title, 80, "Fuse - Write +3 Disk %c:", drive );
+
+  filename = gtkui_fileselector_get_filename( title );
+  if( !filename ) { fuse_emulation_unpause(); return 1; }
+
+  specplus3_disk_write( which, filename );
+
+  free( filename );
+
+  fuse_emulation_unpause();
+
+  return 0;
+}
+#endif				/* #ifdef HAVE_765_H */  
+
+int
+ui_trdos_disk_write( trdos_drive_number which )
+{
+  char drive, *filename, title[80];
+
+  drive = which == TRDOS_DRIVE_A ? 'A' : 'B';
+
+  fuse_emulation_pause();
+
+  snprintf( title, 80, "Fuse - Write TR-DOS Disk %c:", drive );
+
+  filename = gtkui_fileselector_get_filename( title );
+  if( !filename ) { fuse_emulation_unpause(); return 1; }
+
+  trdos_disk_write( which, filename );
+
+  free( filename );
+
+  fuse_emulation_unpause();
+
+  return 0;
 }
 
 /* Cartridge/Insert */
@@ -1076,88 +1170,10 @@ gtkui_destroy_widget_and_quit( GtkWidget *widget, gpointer data GCC_UNUSED )
   gtk_main_quit();
 }
 
-/* Bits used for the file selection dialogs */
-
-typedef struct gktui_fileselector_info {
-
-  GtkWidget *selector;
-  gchar *filename;
-
-} gtkui_fileselector_info;
-
-char*
-gtkui_fileselector_get_filename( const char *title )
-{
-  gtkui_fileselector_info selector;
-  GtkAccelGroup *accel_group;
-
-  selector.selector = gtk_file_selection_new( title );
-  selector.filename = NULL;
-
-  gtk_signal_connect(
-      GTK_OBJECT( GTK_FILE_SELECTION( selector.selector )->ok_button ),
-      "clicked",
-      GTK_SIGNAL_FUNC( gtkui_fileselector_done ),
-      (gpointer) &selector );
-
-  gtk_signal_connect(
-       GTK_OBJECT( GTK_FILE_SELECTION( selector.selector )->cancel_button),
-       "clicked",
-       GTK_SIGNAL_FUNC(gtkui_fileselector_cancel ),
-       (gpointer) &selector );
-
-  gtk_signal_connect( GTK_OBJECT( selector.selector ), "delete_event",
-		      GTK_SIGNAL_FUNC( gtkui_destroy_widget_and_quit ),
-		      (gpointer) &selector );
-
-  accel_group = gtk_accel_group_new();
-  gtk_window_add_accel_group(GTK_WINDOW(selector.selector), accel_group);
-
-  /* Allow Esc to cancel */
-  gtk_widget_add_accelerator(
-       GTK_FILE_SELECTION( selector.selector )->cancel_button,
-       "clicked",
-       accel_group,
-       GDK_Escape, 0, 0 );
-
-  gtk_window_set_modal( GTK_WINDOW( selector.selector ), TRUE );
-
-  gtk_widget_show( selector.selector );
-
-  gtk_main();
-
-  return selector.filename;
-}
-
-static void
-gtkui_fileselector_done( GtkButton *button GCC_UNUSED, gpointer user_data )
-{
-  gtkui_fileselector_info *ptr = (gtkui_fileselector_info*) user_data;
-  const char *filename;
-
-  filename =
-    gtk_file_selection_get_filename( GTK_FILE_SELECTION( ptr->selector ) );
-
-  /* FIXME: what to do if this runs out of memory? */
-  ptr->filename = strdup( filename );
-
-  gtk_widget_destroy( ptr->selector );
-
-  gtk_main_quit();
-}
-
-static void
-gtkui_fileselector_cancel( GtkButton *button GCC_UNUSED, gpointer user_data )
-{
-  gtkui_fileselector_info *ptr = (gtkui_fileselector_info*) user_data;
-
-  gtk_widget_destroy( ptr->selector );
-
-  gtk_main_quit();
-}
-
 /* Functions to activate and deactivate certain menu items */
 
+
+
 static int
 set_menu_item_active( const char *path, int active )
 {
@@ -1182,61 +1198,110 @@ set_menu_item_active( const char *path, int active )
   return 0;
 }
 
-int
-ui_menu_activate_media_cartridge( int active )
+int ui_menu_activate( ui_menu_item item, int active )
 {
-  return set_menu_item_active( "/Media/Cartridge", active );
-}
+  int error;
 
-int
-ui_menu_activate_media_cartridge_eject( int active )
-{
-  return set_menu_item_active( "/Media/Cartridge/Eject", active );
-}
+  switch( item ) {
 
-int
-ui_menu_activate_media_disk( int active )
-{
-  return set_menu_item_active( "/Media/Disk", active );
-}
+  case UI_MENU_ITEM_MEDIA_CARTRIDGE:
+    return set_menu_item_active( "/Media/Cartridge", active );
+
+  case UI_MENU_ITEM_MEDIA_CARTRIDGE_EJECT:
+    return set_menu_item_active( "/Media/Cartridge/Eject", active );
+
+  case UI_MENU_ITEM_MEDIA_DISK:
+    return set_menu_item_active( "/Media/Disk", active );
+
+  case UI_MENU_ITEM_MEDIA_DISK_A_EJECT:
+    error = set_menu_item_active( "/Media/Disk/Drive A:/Eject", active );
+    if( error ) return error;
+
+    return set_menu_item_active( "/Media/Disk/Drive A:/Eject and write...",
+				 active );
+
+  case UI_MENU_ITEM_MEDIA_DISK_B_EJECT:
+    error = set_menu_item_active( "/Media/Disk/Drive B:/Eject", active );
+    if( error ) return error;
+
+    return set_menu_item_active( "/Media/Disk/Drive B:/Eject and write...",
+				 active );
+
+  case UI_MENU_ITEM_RECORDING:
+    error = set_menu_item_active( "/File/Recording/Record...", !active );
+    if( error ) return error;
+
+    error = set_menu_item_active( "/File/Recording/Record from snapshot...",
+				  !active );
+    if( error ) return error;
+
+    error = set_menu_item_active( "/File/Recording/Play...", !active );
+    if( error ) return error;
+
+    return set_menu_item_active( "/File/Recording/Stop", active );
+
+  case UI_MENU_ITEM_AY_LOGGING:
+    error = set_menu_item_active( "/File/AY Logging/Record...", !active );
+    if( error ) return error;
+
+    return set_menu_item_active( "/File/AY Logging/Stop", active );
 
-int
-ui_menu_activate_media_disk_eject( int which, int active )
-{
-  if( which == 0 ) {
-    return set_menu_item_active( "/Media/Disk/Drive A:/Eject", active );
-  } else {
-    return set_menu_item_active( "/Media/Disk/Drive B:/Eject", active );
   }
+
+  ui_error( UI_ERROR_ERROR, "Attempt to activate Unknown menu item %d",
+	    item );
+  return 1;
 }
 
+/*
+ * Font code
+ */
+
 int
-ui_menu_activate_recording( int active )
+gtkui_get_monospaced_font( gtkui_font *font )
 {
-  int error;
+#ifdef UI_GTK2
 
-  error = set_menu_item_active( "/File/Recording/Record...", !active );
-  if( error ) return error;
+  *font = pango_font_description_from_string( "Monospace 10" );
+  if( !(*font) ) {
+    ui_error( UI_ERROR_ERROR, "couldn't find a monospaced font" );
+    return 1;
+  }
 
-  error = set_menu_item_active( "/File/Recording/Record from snapshot...",
-				!active );
-  if( error ) return error;
+#else				/* #ifdef UI_GTK2 */
 
-  error = set_menu_item_active( "/File/Recording/Play...", !active );
-  if( error ) return error;
+  *font = gtk_style_new();
+  gdk_font_unref( (*font)->font );
 
-  return set_menu_item_active( "/File/Recording/Stop", active );
+  (*font)->font = gdk_font_load( "-*-courier-medium-r-*-*-12-*-*-*-*-*-*-*" );
+  if( !(*font)->font ) {
+    ui_error( UI_ERROR_ERROR, "couldn't find a monospaced font" );
+    return 1;
+  }
+
+#endif				/* #ifdef UI_GTK2 */
+
+  return 0;
 }
 
-int
-ui_menu_activate_ay_logging( int active )
+void
+gtkui_free_font( gtkui_font font )
 {
-  int error;
-
-  error = set_menu_item_active( "/File/AY Logging/Record...", !active );
-  if( error ) return error;
-
-  return set_menu_item_active( "/File/AY Logging/Stop", active );
+#ifdef UI_GTK2
+  pango_font_description_free( font );
+#else				/* #ifdef UI_GTK2 */
+  gtk_style_detach( font );
+#endif				/* #ifdef UI_GTK2 */
 }
 
+void
+gtkui_set_font( GtkWidget *widget, gtkui_font font )
+{
+#ifdef UI_GTK2
+  gtk_widget_modify_font( widget, font );
+#else				/* #ifdef UI_GTK2 */
+  gtk_widget_set_style( widget, font );
+#endif				/* #ifdef UI_GTK2 */
+}  
+
 #endif			/* #ifdef UI_GTK */
