@@ -1,7 +1,7 @@
 /* svgakeyboard.c: svgalib routines for dealing with the keyboard
-   Copyright (c) 2000-2002 Philip Kendall, Matan Ziv-Av
+   Copyright (c) 2000-2004 Philip Kendall, Matan Ziv-Av
 
-   $Id: svgakeyboard.c,v 1.44 2003/02/09 12:06:53 pak21 Exp $
+   $Id: svgakeyboard.c,v 1.45.2.1 2004/02/07 10:52:26 pak21 Exp $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -41,6 +41,7 @@
 #include "snapshot.h"
 #include "spectrum.h"
 #include "tape.h"
+#include "utils.h"
 #ifdef USE_WIDGET
 #include "widget/widget.h"
 #endif				/* #ifdef USE_WIDGET */
@@ -97,7 +98,12 @@ static void svgakeyboard_keypress(int keysym)
     break;
   case SCANCODE_F3:
     fuse_emulation_pause();
-    widget_apply_to_file( snapshot_read );
+    widget_do( WIDGET_TYPE_FILESELECTOR, NULL );
+    if( widget_filesel_name ) {
+      utils_open_file( widget_filesel_name, settings_current.auto_load, NULL );
+      free( widget_filesel_name );
+      display_refresh_all();
+    }
     fuse_emulation_unpause();
     break;
   case SCANCODE_F4:
