@@ -1,7 +1,7 @@
 /* xdisplay.c: Routines for dealing with drawing the Speccy's screen via Xlib
    Copyright (c) 2000,2002 Philip Kendall, Darren Salt
 
-   $Id: xdisplay.c,v 1.27 2002/03/15 17:08:04 pak21 Exp $
+   $Id: xdisplay.c,v 1.48 2003/07/11 11:15:18 pak21 Exp $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -51,18 +51,39 @@
 #include <X11/extensions/XShm.h>
 #endif				/* #ifdef X_USE_SHM */
 
+#include <libspectrum.h>
+
 #include "display.h"
 #include "fuse.h"
 #include "keyboard.h"
+#include "machine.h"
+#include "screenshot.h"
 #include "xdisplay.h"
 #include "xui.h"
+#include "ui/scaler/scaler.h"
 #include "ui/uidisplay.h"
+#ifdef USE_WIDGET
 #include "widget/widget.h"
+#endif				/* #ifdef USE_WIDGET */
+#include "scld.h"
 
 static XImage *image = 0;	/* The image structure to draw the
 				   Speccy's screen on */
 static GC gc;			/* A graphics context to draw with */
 
+/* The size of a 1x1 image in units of
+   DISPLAY_ASPECT WIDTH x DISPLAY_SCREEN_HEIGHT */
+int image_scale;
+
+/* The height and width of a 1x1 image in pixels */
+int image_width, image_height;
+
+/* A scaled copy of the image displayed on the Spectrum's screen */
+static libspectrum_word
+  scaled_image[ 2 * DISPLAY_SCREEN_HEIGHT ][ 2 * DISPLAY_SCREEN_WIDTH ];
+static const ptrdiff_t scaled_pitch =
+  2 * DISPLAY_SCREEN_WIDTH * sizeof( libspectrum_word );
+
 static unsigned long colours[16];
 
 /* The current size of the window (in units of DISPLAY_SCREEN_*) */
@@ -79,27 +100,33 @@ static int get_shm_id( const int size );
 
 static int shm_used = 0;
 
-static int xdisplay_allocate_colours(int numColours, unsigned long *colours);
-static int xdisplay_allocate_gc(Window window, GC *gc);
-static int xdisplay_allocate_image(int width, int height);
-static void xdisplay_destroy_image( void );
-static void xdisplay_end( int );
+static int xdisplay_allocate_colours( int numColours,
+				      unsigned long *colour_values );
+static int xdisplay_allocate_gc( Window window, GC *new_gc );
 
-int uidisplay_init(int width, int height)
+static int xdisplay_allocate_image(int width, int height);
+static int register_scalers( void );
+static void xdisplay_destroy_image( void );
+static void xdisplay_catch_signal( int sig );
+
+int
+xdisplay_init( void )
 {
   if(xdisplay_allocate_colours(16,colours)) return 1;
   if(xdisplay_allocate_gc(xui_mainWindow,&gc)) return 1;
-  if(xdisplay_allocate_image(width,height)) return 1;
+  if( xdisplay_allocate_image( DISPLAY_ASPECT_WIDTH, DISPLAY_SCREEN_HEIGHT ) )
+     return 1;
 
   return 0;
 }
 
-static int xdisplay_allocate_colours(int numColours, unsigned long *colours)
+static int
+xdisplay_allocate_colours( int numColours, unsigned long *colour_values )
 {
   XColor colour;
   Colormap currentMap;
 
-  char *colour_names[] = {
+  const char *colour_names[] = {
     "black",
     "blue3",
     "red3",
@@ -140,18 +167,19 @@ static int xdisplay_allocate_colours(int numColours, unsigned long *colours)
 	return 1;
       }
     }
-    colours[i]=colour.pixel;
+    colour_values[i] = colour.pixel;
   }
 
   return 0;
 }
   
-static int xdisplay_allocate_gc(Window window,GC *gc)
+static int
+xdisplay_allocate_gc( Window window, GC *new_gc )
 {
   unsigned valuemask=0;
   XGCValues values;
 
-  *gc=XCreateGC(display,window,valuemask,&values);
+  *new_gc = XCreateGC( display, window, valuemask, &values );
 
   return 0;
 }
@@ -160,7 +188,7 @@ static int xdisplay_allocate_image(int width, int height)
 {
   struct sigaction handler;
 
-  handler.sa_handler = xdisplay_end;
+  handler.sa_handler = xdisplay_catch_signal;
   sigemptyset( &handler.sa_mask );
   handler.sa_flags = 0;
   sigaction( SIGINT, &handler, NULL );
@@ -287,11 +315,72 @@ get_shm_id( const int size )
 }
 #endif			/* #ifdef X_USE_SHM */
 
-int xdisplay_configure_notify(int width, int height)
+int
+uidisplay_init( int width, int height )
 {
-  int y,size;
+  int error;
 
-  size = width / DISPLAY_SCREEN_WIDTH;
+  image_width = width; image_height = height;
+  image_scale = width / DISPLAY_ASPECT_WIDTH;
+
+  error = register_scalers(); if( error ) return error;
+
+  display_ui_initialised = 1;
+
+  display_refresh_all();
+
+  return 0;
+}
+
+static int
+register_scalers( void )
+{
+  scaler_register_clear();
+
+  switch( xdisplay_current_size ) {
+
+  case 1:
+
+    switch( image_scale ) {
+    case 1:
+      scaler_register( SCALER_NORMAL );
+      scaler_select_scaler( SCALER_NORMAL );
+      return 0;
+    case 2:
+      scaler_register( SCALER_HALFSKIP );
+      scaler_select_scaler( SCALER_HALFSKIP );
+      return 0;
+    }
+
+  case 2:
+
+    switch( image_scale ) {
+    case 1:
+      scaler_register( SCALER_DOUBLESIZE );
+      scaler_register( SCALER_ADVMAME2X );
+      scaler_select_scaler( SCALER_DOUBLESIZE );
+      return 0;
+    case 2:
+      scaler_register( SCALER_NORMAL );
+      scaler_select_scaler( SCALER_NORMAL );
+      return 0;
+    }
+
+  }
+
+  ui_error( UI_ERROR_ERROR, "Unknown display size/image size %d/%d",
+	    xdisplay_current_size, image_scale );
+  return 1;
+}
+
+int
+xdisplay_configure_notify( int width, int height GCC_UNUSED )
+{
+  int error, size, colour;
+
+  colour = scld_last_dec.name.hires ? display_hires_border :
+                                      display_lores_border;
+  size = width / DISPLAY_ASPECT_WIDTH;
 
   /* If we're the same size as before, nothing special needed */
   if( size == xdisplay_current_size ) return 0;
@@ -299,102 +388,65 @@ int xdisplay_configure_notify(int width, int height)
   /* Else set ourselves to the new height */
   xdisplay_current_size=size;
 
+  /* Get a new scaler */
+  error = register_scalers(); if( error ) return error;
+
   /* Redraw the entire screen... */
   display_refresh_all();
 
-  /* And the entire border */
-  for(y=0;y<DISPLAY_BORDER_HEIGHT;y++) {
-    uidisplay_set_border(y, 0, DISPLAY_SCREEN_WIDTH, display_border);
-    uidisplay_set_border(DISPLAY_BORDER_HEIGHT+DISPLAY_HEIGHT+y, 0,
-			 DISPLAY_SCREEN_WIDTH, display_border);
-  }
-
-  for(y=DISPLAY_BORDER_HEIGHT;y<DISPLAY_BORDER_HEIGHT+DISPLAY_HEIGHT;y++) {
-    uidisplay_set_border(y, 0, DISPLAY_BORDER_WIDTH, display_border);
-    uidisplay_set_border(y, DISPLAY_BORDER_WIDTH+DISPLAY_WIDTH,
-			 DISPLAY_SCREEN_WIDTH, display_border);
-  }
+  /* If widgets are active, redraw the widget */
+  if( widget_level >= 0 ) widget_keyhandler( KEYBOARD_Resize, KEYBOARD_NONE );
 
   return 0;
 }
 
-void uidisplay_putpixel(int x,int y,int colour)
+void
+uidisplay_frame_end( void ) 
 {
-  switch(xdisplay_current_size) {
-  case 1:
-    XPutPixel(image,  x  ,  y  ,colours[colour]);
-    break;
-  case 2:
-    XPutPixel(image,2*x,  2*y  ,colours[colour]);
-    XPutPixel(image,2*x+1,2*y  ,colours[colour]);
-    XPutPixel(image,2*x  ,2*y+1,colours[colour]);
-    XPutPixel(image,2*x+1,2*y+1,colours[colour]);
-    break;
-  case 3:
-    XPutPixel(image,3*x,  3*y  ,colours[colour]);
-    XPutPixel(image,3*x+1,3*y  ,colours[colour]);
-    XPutPixel(image,3*x+2,3*y  ,colours[colour]);
-    XPutPixel(image,3*x  ,3*y+1,colours[colour]);
-    XPutPixel(image,3*x+1,3*y+1,colours[colour]);
-    XPutPixel(image,3*x+2,3*y+1,colours[colour]);
-    XPutPixel(image,3*x  ,3*y+2,colours[colour]);
-    XPutPixel(image,3*x+1,3*y+2,colours[colour]);
-    XPutPixel(image,3*x+2,3*y+2,colours[colour]);
-    break;
-  }
+  return;
 }
 
-void uidisplay_lines( int start, int end )
+void
+uidisplay_area( int x, int y, int w, int h )
 {
-  xdisplay_area( 0, xdisplay_current_size * start,
-		 xdisplay_current_size * DISPLAY_SCREEN_WIDTH,
-		 xdisplay_current_size * ( end - start + 1 ) );
+  float scale = (float)xdisplay_current_size / image_scale;
+  int scaled_x, scaled_y, xx, yy;
+
+  /* Extend the dirty region by 1 pixel for scalers
+       that "smear" the screen, e.g. 2xSAI */
+  if( scaler_flags & SCALER_FLAGS_EXPAND )
+    scaler_expander( &x, &y, &w, &h, image_width, image_height );
+
+  scaled_x = scale * x; scaled_y = scale * y;
+
+  /* Create scaled image */
+  scaler_proc16( (libspectrum_byte*)&display_image[y][x], display_pitch,
+		 (libspectrum_byte*)&scaled_image[scaled_y][scaled_x],
+		 scaled_pitch, w, h );
+
+  w *= scale; h *= scale;
+
+  /* Call putpixel multiple times */
+  for( yy = scaled_y; yy < scaled_y + h; yy++ )
+    for( xx = scaled_x; xx < scaled_x + w; xx++ ) {
+      int colour = scaled_image[yy][xx];
+      XPutPixel( image, xx, yy, colours[ colour ] );
+    }
+
+  /* Blit to the real screen */
+  xdisplay_area( scaled_x, scaled_y, w, h );
 }
 
-void xdisplay_area(int x, int y, int width, int height)
+void
+xdisplay_area( int x, int y, int w, int h )
 {
   if( shm_used ) {
 #ifdef X_USE_SHM
-    XShmPutImage( display, xui_mainWindow, gc, image,
-		  x, y, x, y, width, height, True );
+    XShmPutImage( display, xui_mainWindow, gc, image, x, y, x, y, w, h, True );
     /* FIXME: should wait for an ShmCompletion event here */
 #endif				/* #ifdef X_USE_SHM */
   } else {
-    XPutImage(display, xui_mainWindow, gc, image, x, y, x, y, width, height);
-  }
-}
-
-void uidisplay_set_border(int line, int pixel_from, int pixel_to, int colour)
-{
-  int x;
-  
-  switch(xdisplay_current_size) {
-  case 1:
-    for(x=pixel_from;x<pixel_to;x++) {
-      XPutPixel(image,  x  ,  line  ,colours[colour]);
-    }
-    break;
-  case 2:
-    for(x=pixel_from;x<pixel_to;x++) {
-      XPutPixel(image,2*x  ,2*line  ,colours[colour]);
-      XPutPixel(image,2*x+1,2*line  ,colours[colour]);
-      XPutPixel(image,2*x  ,2*line+1,colours[colour]);
-      XPutPixel(image,2*x+1,2*line+1,colours[colour]);
-    }
-    break;
-  case 3:
-    for(x=pixel_from;x<pixel_to;x++) {
-      XPutPixel(image,3*x  ,3*line  ,colours[colour]);
-      XPutPixel(image,3*x+1,3*line  ,colours[colour]);
-      XPutPixel(image,3*x+2,3*line  ,colours[colour]);
-      XPutPixel(image,3*x  ,3*line+1,colours[colour]);
-      XPutPixel(image,3*x+1,3*line+1,colours[colour]);
-      XPutPixel(image,3*x+2,3*line+1,colours[colour]);
-      XPutPixel(image,3*x  ,3*line+2,colours[colour]);
-      XPutPixel(image,3*x+1,3*line+2,colours[colour]);
-      XPutPixel(image,3*x+2,3*line+2,colours[colour]);
-    }
-    break;
+    XPutImage( display, xui_mainWindow, gc, image, x, y, x, y, w, h );
   }
 }
 
@@ -413,14 +465,29 @@ static void xdisplay_destroy_image (void)
   if( image ) XDestroyImage( image ); image = 0;
 }
 
-static void xdisplay_end (int sig)
+void
+uidisplay_hotswap_gfx_mode( void )
 {
-  uidisplay_end();
+  return;
+}
+
+int
+uidisplay_end( void )
+{
+  display_ui_initialised = 0;
+  return 0;
+}
+
+static void
+xdisplay_catch_signal( int sig )
+{
+  xdisplay_end();
   psignal( sig, fuse_progname );
   exit( 1 );
 }
 
-int uidisplay_end(void)
+int
+xdisplay_end( void )
 {
   xdisplay_destroy_image();
   /* Free the allocated GC */
