@@ -1,7 +1,7 @@
 /* z80.h: z80 emulation core
    Copyright (c) 1999-2003 Philip Kendall
 
-   $Id: z80.h,v 1.6 2003/07/11 11:03:25 pak21 Exp $
+   $Id: z80.h,v 1.9 2003/11/11 16:03:20 pak21 Exp $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -49,16 +49,23 @@ typedef struct {
   regpair sp,pc;
   libspectrum_byte iff1, iff2, im;
   int halted;
+
+  /* Interrupts were enabled at this time; do not accept any interrupts
+     until tstates > this value */
+  libspectrum_signed_dword interrupts_enabled_at;
+
 } processor;
 
 void z80_init(void);
 void z80_reset(void);
 
-void z80_interrupt(void);
+int z80_interrupt( void );
 void z80_nmi( void );
 
 void z80_do_opcodes(void);
 
+void z80_enable_interrupts( void );
+
 extern processor z80;
 extern libspectrum_byte halfcarry_add_table[];
 extern libspectrum_byte halfcarry_sub_table[];
